@@ -9,10 +9,14 @@ import shutil
 import sys
 from pathlib import Path
 from typing import Set, Union
-from colored import stylize, fg, bg, attr
+
+from rich.console import Console
+from rich.text import Text
+from rich.theme import Theme
+from natsort import natsorted, ns
 import regex
 
-VERSION = "1.2.4"
+VERSION = "1.5.0"
 MESSAGE_INDENT = "    "
 UNICODE_BOM = "\ufeff"
 NO_BREAK_SPACE = "\u00a0"
@@ -23,13 +27,14 @@ SHY_HYPHEN = "\u00ad"
 FUNCTION_APPLICATION = "\u2061"
 NO_BREAK_HYPHEN = "\u2011"
 IGNORED_FILENAMES = ["colophon.xhtml", "titlepage.xhtml", "imprint.xhtml", "uncopyright.xhtml", "halftitle.xhtml", "toc.xhtml", "loi.xhtml"]
-XHTML_NAMESPACES = {"xhtml": "http://www.w3.org/1999/xhtml", "epub": "http://www.idpf.org/2007/ops", "z3998": "http://www.daisy.org/z3998/2012/vocab/structure/", "se": "https://standardebooks.org/vocab/1.0", "dc": "http://purl.org/dc/elements/1.1/", "opf": "http://www.idpf.org/2007/opf", "container": "urn:oasis:names:tc:opendocument:xmlns:container", "m": "http://www.w3.org/1998/Math/MathML"}
+# The `re` namespace enables regex functions in xpaths in lxml
+XHTML_NAMESPACES = {"xhtml": "http://www.w3.org/1999/xhtml", "epub": "http://www.idpf.org/2007/ops", "z3998": "http://www.daisy.org/z3998/2012/vocab/structure/", "se": "https://standardebooks.org/vocab/1.0", "dc": "http://purl.org/dc/elements/1.1/", "opf": "http://www.idpf.org/2007/opf", "container": "urn:oasis:names:tc:opendocument:xmlns:container", "m": "http://www.w3.org/1998/Math/MathML", "re": "http://exslt.org/regular-expressions"}
 FRONTMATTER_FILENAMES = ["dedication.xhtml", "introduction.xhtml", "preface.xhtml", "foreword.xhtml", "preamble.xhtml", "titlepage.xhtml", "halftitlepage.xhtml", "imprint.xhtml"]
 BACKMATTER_FILENAMES = ["endnotes.xhtml", "loi.xhtml", "afterword.xhtml", "appendix.xhtml", "colophon.xhtml", "uncopyright.xhtml"]
 BINARY_EXTENSIONS = [".jpg", ".jpeg", ".tif", ".tiff", ".bmp", ".png", ".epub", ".epub3", ".xcf", ".otf"]
 SE_GENRES = ["Adventure", "Autobiography", "Biography", "Childrens", "Comedy", "Drama", "Fantasy", "Fiction", "Horror", "Memoir", "Mystery", "Nonfiction", "Philosophy", "Poetry", "Romance", "Satire", "Science Fiction", "Shorts", "Spirituality", "Tragedy", "Travel"]
 ARIA_ROLES = ["afterword", "appendix", "biblioentry", "bibliography", "chapter", "colophon", "conclusion", "dedication", "epilogue", "foreword", "introduction", "noteref", "part", "preface", "prologue", "subtitle", "toc"]
-IGNORED_CLASSES = ["name", "temperature", "state", "era", "compass", "acronym", "postal", "eoc", "initialism", "degree", "time", "compound", "timezone", "signature", "full-page"]
+IGNORED_CLASSES = ["elision", "name", "temperature", "state", "era", "compass", "acronym", "postal", "eoc", "initialism", "degree", "time", "compound", "timezone", "signature", "full-page"]
 SELECTORS_TO_SIMPLIFY = [":first-child", ":only-child", ":last-child", ":nth-child", ":nth-last-child", ":first-of-type", ":only-of-type", ":last-of-type", ":nth-of-type", ":nth-last-of-type"]
 MESSAGE_TYPE_WARNING = 1
 MESSAGE_TYPE_ERROR = 2
@@ -62,6 +67,17 @@ LEAGUE_SPARTAN_AVERAGE_SPACING = 7 # Guess at average default spacing between le
 LEAGUE_SPARTAN_100_WIDTHS = {" ": 40.0, "A": 98.245, "B": 68.1875, "C": 83.97625, "D": 76.60875, "E": 55.205, "F": 55.79, "G": 91.57875, "H": 75.0875, "I": 21.98875, "J": 52.631254, "K": 87.83625, "L": 55.205, "M": 106.9, "N": 82.5725, "O": 97.1925, "P": 68.1875, "Q": 98.83, "R": 79.41599, "S": 72.63125, "T": 67.83625, "U": 75.32125, "V": 98.245, "W": 134.62, "X": 101.28625, "Y": 93.1, "Z": 86.19875, ".": 26.78375, ",": 26.78375, "/": 66.08125, "\\": 66.08125, "-": 37.66125, ":": 26.78375, ";": 26.78375, "’": 24.3275, "!": 26.78375, "?": 64.3275, "&": 101.87125, "0": 78.48, "1": 37.895, "2": 75.205, "3": 72.04625, "4": 79.29875, "5": 70.175, "6": 74.26875, "7": 76.95875, "8": 72.16375, "9": 74.26875}
 NOVELLA_MIN_WORD_COUNT = 17500
 NOVEL_MIN_WORD_COUNT = 40000
+RICH_THEME = Theme({
+	"xhtml": "bright_blue",
+	"xml": "bright_blue",
+	"val": "bright_blue",
+	"attr": "bright_blue",
+	"class": "bright_blue",
+	"path": "bright_blue",
+	"url": "bright_blue",
+	"text": "bright_blue",
+	"bash": "bright_blue"
+})
 
 class SeException(Exception):
 	""" Wrapper class for SE exceptions """
@@ -103,10 +119,9 @@ class InvalidSeEbookException(SeException):
 	""" Invalid SE ebook """
 	code = 10
 
-# No longer used--code 11 can be replaced with another exception
-# class FirefoxRunningException(SeException):
-# 	""" Firefox already running """
-# 	code = 11
+class InvalidArgumentsException(SeException):
+	""" Invalid arguments """
+	code = 11
 
 class RemoteCommandErrorException(SeException):
 	""" Error in remote command """
@@ -120,43 +135,17 @@ class InvalidCssException(SeException):
 	""" Invalid CSS """
 	code = 14
 
-def natural_sort(list_to_sort: list) -> list:
-	"""
-	Natural sort a list.
-	"""
+class InvalidSvgException(SeException):
+	""" Invalid SVG """
+	code = 15
 
-	convert = lambda text: int(text) if text.isdigit() else text.lower()
-	alphanum_key = lambda key: [convert(c) for c in regex.split("([0-9]+)", str(key))]
+class InvalidXmlException(SeException):
+	""" Invalid XHTML """
+	code = 16
 
-	return sorted(list_to_sort, key=alphanum_key)
-
-def natural_sort_key(text: str, _nsre=regex.compile("([0-9]+)")):
-	"""
-	Helper function for sorted() to sort by key.
-	"""
-
-	return [int(text) if text.isdigit() else text.lower() for text in regex.split(_nsre, text)]
-
-def replace_in_file(file_path: Path, search: Union[str, list], replace: Union[str, list]) -> None:
-	"""
-	Helper function to replace in a file.
-	"""
-
-	with open(file_path, "r+", encoding="utf-8") as file:
-		data = file.read()
-		processed_data = data
-
-		if isinstance(search, list):
-			for index, val in enumerate(search):
-				if replace[index] is not None:
-					processed_data = processed_data.replace(val, replace[index])
-		else:
-			processed_data = processed_data.replace(search, str(replace))
-
-		if processed_data != data:
-			file.seek(0)
-			file.write(processed_data)
-			file.truncate()
+class BuildFailedException(SeException):
+	""" Build failed """
+	code = 17
 
 def strip_bom(string: str) -> str:
 	"""
@@ -184,26 +173,37 @@ def quiet_remove(file: Path) -> None:
 	except Exception:
 		pass
 
-def print_error(message: Union[SeException, str], verbose: bool = False) -> None:
+def print_error(message: Union[SeException, str], verbose: bool = False, is_warning: bool = False) -> None:
 	"""
 	Helper function to print a colored error message to the console.
+
+	Allowed BBCode values:
+	[link=foo]bar[/] - Hyperlink
+	[xml] - XML, usually a tag
+	[xhtml] - XHTML, usually a tag
+	[attr] - A lone XHTML attribute
+	[val] - A lone XHTML attribute value (not a class)
+	[class] - A lone XHTML class value
+	[path] - Filesystem path or glob
+	[url] - A URL
+	[text] - Non-semantic text that requires color
+	[bash] - A command or flag of a command
 	"""
 
-	if isinstance(message, SeException):
-		message = str(message)
+	label = "Error" if not is_warning else "Warning"
+	bg_color = 'red' if not is_warning else 'yellow'
 
-	# By convention, any text within the message text that is surrounded in backticks
-	# is rendered in blue
-	message = regex.sub(r"`(.+?)`", stylize(r"\1", fg("light_blue")), message)
+	# We have to print to stdout in case we're called from GNU Parallel, otherwise weird newline issues occur
+	# This no longer works with rich because it can't (yet) output to stderr
+	output_file = sys.stderr if not is_warning and not is_called_from_parallel() else sys.stdout
 
-	print("{}{} {}".format(MESSAGE_INDENT if verbose else "", stylize(" Error ", bg("red") + attr("bold")), message), file=sys.stderr)
+	message = str(message)
 
-def print_warning(message: str, verbose: bool = False) -> None:
-	"""
-	Helper function to print a colored warning message to the console.
-	"""
+	if verbose:
+		message = str(message).replace("\n", f"\n{MESSAGE_INDENT}")
 
-	print("{}{} {}".format(MESSAGE_INDENT if verbose else "", stylize(" Warning ", bg("yellow") + attr("bold")), message))
+	console = Console(file=output_file, highlight=False, theme=RICH_THEME, force_terminal=is_called_from_parallel()) # Syntax highlighting will do weird things when printing paths; force_terminal prints colors when called from GNU Parallel
+	console.print(f"{MESSAGE_INDENT if verbose else ''}[white on {bg_color} bold] {label} [/] {message}")
 
 def is_positive_integer(value: str) -> int:
 	"""
@@ -253,23 +253,22 @@ def get_target_filenames(targets: list, allowed_extensions: tuple, ignored_filen
 			if target.name.endswith(allowed_extensions):
 				target_xhtml_filenames.add(target)
 
-	return natural_sort(list(target_xhtml_filenames))
+	return natsorted(list(target_xhtml_filenames), key=lambda x: str(x.name), alg=ns.PATH)
 
-def get_xhtml_language(xhtml: str) -> str:
+def is_called_from_parallel() -> bool:
 	"""
-	Try to get the IETF lang tag for a complete XHTML document
+	Decide if we're being called from GNU parallel.
+	This is good to know in case we want to tweak some output.
 	"""
 
-	supported_languages = ["en-US", "en-GB", "en-AU", "en-CA"]
+	import psutil # pylint: disable=import-outside-toplevel
 
-	match = regex.search(r"<html[^>]+?xml:lang=\"([^\"]+)\"", xhtml)
+	try:
+		for line in psutil.Process(psutil.Process().ppid()).cmdline():
+			if regex.search(fr"{os.sep}parallel$", line):
+				return True
+	except:
+		# If we can't figure it out, don't worry about it
+		pass
 
-	if match:
-		language = match.group(1)
-	else:
-		language = None
-
-	if language not in supported_languages:
-		raise InvalidLanguageException("No valid xml:lang attribute in <html> root. Only {} are supported.".format(", ".join(supported_languages[:-1]) + ", and " + supported_languages[-1]))
-
-	return language
+	return False
