@@ -19,7 +19,28 @@ from ftfy import fix_text
 
 import se
 import se.formatting
+from se.se_epub import SeEpub
 
+def _replace_in_file(file_path: Path, search: Union[str, list], replace: Union[str, list]) -> None:
+	"""
+	Helper function to replace in a file.
+	"""
+
+	with open(file_path, "r+", encoding="utf-8") as file:
+		data = file.read()
+		processed_data = data
+
+		if isinstance(search, list):
+			for index, val in enumerate(search):
+				if replace[index] is not None:
+					processed_data = processed_data.replace(val, replace[index])
+		else:
+			processed_data = processed_data.replace(search, str(replace))
+
+		if processed_data != data:
+			file.seek(0)
+			file.write(processed_data)
+			file.truncate()
 
 def _get_word_widths(string: str, target_height: int) -> list:
 	"""
@@ -87,6 +108,14 @@ def _calculate_image_lines(string: str, target_height: int, canvas_width: int) -
 
 	lines.reverse()
 
+	# If the first line is a single short word, move up the first word of the next line
+	if lines[0].lower() in ("the", "a", "of", "by", "on"):
+		first_word = regex.match(r"^[\p{Letter}]+(?=\s)", lines[1])
+
+		if first_word:
+			lines[0] = lines[0] + " " + first_word.group(0)
+			lines[1] = regex.sub(rf"^{regex.escape(first_word.group(0))}", "", lines[1])
+
 	return lines
 
 def _generate_titlepage_svg(title: str, authors: Union[str, list], contributors: dict, title_string: str) -> str:
@@ -298,7 +327,7 @@ def _get_wikipedia_url(string: str, get_nacoaf_url: bool) -> Tuple[Optional[str]
 	try:
 		response = requests.get("https://en.wikipedia.org/wiki/Special:Search", params={"search": string, "go": "Go", "ns0": "1"}, allow_redirects=False)
 	except Exception as ex:
-		se.print_error(f"Couldn’t contact Wikipedia. Error: {ex}")
+		se.print_error(f"Couldn’t contact Wikipedia. Exception: {ex}")
 
 	if response.status_code == 302:
 		nacoaf_url = None
@@ -311,9 +340,9 @@ def _get_wikipedia_url(string: str, get_nacoaf_url: bool) -> Tuple[Optional[str]
 			try:
 				response = requests.get(wiki_url)
 			except Exception as ex:
-				se.print_error(f"Couldn’t contact Wikipedia. Error: {ex}")
+				se.print_error(f"Couldn’t contact Wikipedia. Exception: {ex}")
 
-			for match in regex.findall(r"http://id\.loc\.gov/authorities/names/n[0-9]+", response.text):
+			for match in regex.findall(r"https?://id\.loc\.gov/authorities/names/n[0-9]+", response.text):
 				nacoaf_url = match
 
 		return wiki_url, nacoaf_url
@@ -346,15 +375,18 @@ def _create_draft(args: Namespace):
 		identifier = identifier + "/" + se.formatting.make_url_safe(args.illustrator)
 		title_string = title_string + ". Illustrated by " + args.illustrator
 
-	repo_name = Path(identifier.replace("/", "_"))
+	repo_name = identifier.replace("/", "_")
 
-	if repo_name.is_dir():
-		raise se.InvalidInputException(f"`./{repo_name}/` already exists.")
+	repo_path = Path(repo_name).resolve()
+
+	if repo_path.is_dir():
+		raise se.InvalidInputException(f"Directory already exists: [path][link=file://{repo_path}]{repo_path}[/][/].")
 
 	# Download PG HTML and do some fixups
 	if args.pg_url:
 		if args.offline:
 			raise se.RemoteCommandErrorException("Cannot download Project Gutenberg ebook when offline option is enabled.")
+
 		args.pg_url = args.pg_url.replace("http://", "https://")
 
 		# Get the ebook metadata
@@ -362,7 +394,7 @@ def _create_draft(args: Namespace):
 			response = requests.get(args.pg_url)
 			pg_metadata_html = response.text
 		except Exception as ex:
-			raise se.RemoteCommandErrorException(f"Couldn’t download Project Gutenberg ebook metadata page. Error: {ex}")
+			raise se.RemoteCommandErrorException(f"Couldn’t download Project Gutenberg ebook metadata page. Exception: {ex}")
 
 		soup = BeautifulSoup(pg_metadata_html, "lxml")
 
@@ -392,13 +424,13 @@ def _create_draft(args: Namespace):
 			response = requests.get(pg_ebook_url)
 			pg_ebook_html = response.text
 		except Exception as ex:
-			raise se.RemoteCommandErrorException(f"Couldn’t download Project Gutenberg ebook HTML. Error: {ex}")
+			raise se.RemoteCommandErrorException(f"Couldn’t download Project Gutenberg ebook HTML. Exception: {ex}")
 
 		try:
 			fixed_pg_ebook_html = fix_text(pg_ebook_html, uncurl_quotes=False)
 			pg_ebook_html = se.strip_bom(fixed_pg_ebook_html)
 		except Exception as ex:
-			raise se.InvalidEncodingException(f"Couldn’t determine text encoding of Project Gutenberg HTML file. Error: {ex}")
+			raise se.InvalidEncodingException(f"Couldn’t determine text encoding of Project Gutenberg HTML file. Exception: {ex}")
 
 		# Try to guess the ebook language
 		pg_language = "en-US"
@@ -406,11 +438,11 @@ def _create_draft(args: Namespace):
 			pg_language = "en-GB"
 
 	# Create necessary directories
-	(repo_name / "images").mkdir(parents=True)
-	(repo_name / "src" / "epub" / "css").mkdir(parents=True)
-	(repo_name / "src" / "epub" / "images").mkdir(parents=True)
-	(repo_name / "src" / "epub" / "text").mkdir(parents=True)
-	(repo_name / "src" / "META-INF").mkdir(parents=True)
+	(repo_path / "images").mkdir(parents=True)
+	(repo_path / "src" / "epub" / "css").mkdir(parents=True)
+	(repo_path / "src" / "epub" / "images").mkdir(parents=True)
+	(repo_path / "src" / "epub" / "text").mkdir(parents=True)
+	(repo_path / "src" / "META-INF").mkdir(parents=True)
 
 	is_pg_html_parsed = True
 
@@ -446,35 +478,35 @@ def _create_draft(args: Namespace):
 
 				element.parent.decompose()
 
-			with open(repo_name / "src" / "epub" / "text" / "body.xhtml", "w", encoding="utf-8") as file:
+			with open(repo_path / "src" / "epub" / "text" / "body.xhtml", "w", encoding="utf-8") as file:
 				file.write(str(soup))
 		except OSError as ex:
-			raise se.InvalidFileException(f"Couldn’t write to ebook directory. Error: {ex}")
+			raise se.InvalidFileException(f"Couldn’t write to ebook directory. Exception: {ex}")
 		except:
 			# Save this error for later, because it's still useful to complete the create-draft process
 			# even if we've failed to parse PG's HTML source.
 			is_pg_html_parsed = False
-			se.quiet_remove(repo_name / "src" / "epub" / "text" / "body.xhtml")
+			se.quiet_remove(repo_path / "src" / "epub" / "text" / "body.xhtml")
 
 	# Copy over templates
 
-	_copy_template_file("gitignore", repo_name / ".gitignore")
-	_copy_template_file("LICENSE.md", repo_name)
-	_copy_template_file("container.xml", repo_name / "src" / "META-INF")
-	_copy_template_file("mimetype", repo_name / "src")
-	_copy_template_file("content.opf", repo_name / "src" / "epub")
-	_copy_template_file("onix.xml", repo_name / "src" / "epub")
-	_copy_template_file("toc.xhtml", repo_name / "src" / "epub")
-	_copy_template_file("core.css", repo_name / "src" / "epub" / "css")
-	_copy_template_file("local.css", repo_name / "src" / "epub" / "css")
-	_copy_template_file("logo.svg", repo_name / "src" / "epub" / "images")
-	_copy_template_file("colophon.xhtml", repo_name / "src" / "epub" / "text")
-	_copy_template_file("imprint.xhtml", repo_name / "src" / "epub" / "text")
-	_copy_template_file("titlepage.xhtml", repo_name / "src" / "epub" / "text")
-	_copy_template_file("uncopyright.xhtml", repo_name / "src" / "epub" / "text")
-	_copy_template_file("titlepage.svg", repo_name / "images")
-	_copy_template_file("cover.jpg", repo_name / "images" / "cover.jpg")
-	_copy_template_file("cover.svg", repo_name / "images" / "cover.svg")
+	_copy_template_file("gitignore", repo_path / ".gitignore")
+	_copy_template_file("LICENSE.md", repo_path)
+	_copy_template_file("container.xml", repo_path / "src" / "META-INF")
+	_copy_template_file("mimetype", repo_path / "src")
+	_copy_template_file("content.opf", repo_path / "src" / "epub")
+	_copy_template_file("onix.xml", repo_path / "src" / "epub")
+	_copy_template_file("toc.xhtml", repo_path / "src" / "epub")
+	_copy_template_file("core.css", repo_path / "src" / "epub" / "css")
+	_copy_template_file("local.css", repo_path / "src" / "epub" / "css")
+	_copy_template_file("logo.svg", repo_path / "src" / "epub" / "images")
+	_copy_template_file("colophon.xhtml", repo_path / "src" / "epub" / "text")
+	_copy_template_file("imprint.xhtml", repo_path / "src" / "epub" / "text")
+	_copy_template_file("titlepage.xhtml", repo_path / "src" / "epub" / "text")
+	_copy_template_file("uncopyright.xhtml", repo_path / "src" / "epub" / "text")
+	_copy_template_file("titlepage.svg", repo_path / "images")
+	_copy_template_file("cover.jpg", repo_path / "images" / "cover.jpg")
+	_copy_template_file("cover.svg", repo_path / "images" / "cover.svg")
 
 	# Try to find Wikipedia links if possible
 	if args.offline:
@@ -494,9 +526,9 @@ def _create_draft(args: Namespace):
 			translator_wiki_url, translator_nacoaf_url = _get_wikipedia_url(args.translator, True)
 
 	# Pre-fill a few templates
-	se.replace_in_file(repo_name / "src" / "epub" / "text" / "titlepage.xhtml", "TITLE_STRING", title_string)
-	se.replace_in_file(repo_name / "images" / "titlepage.svg", "TITLE_STRING", title_string)
-	se.replace_in_file(repo_name / "images" / "cover.svg", "TITLE_STRING", title_string)
+	_replace_in_file(repo_path / "src" / "epub" / "text" / "titlepage.xhtml", "TITLE_STRING", title_string)
+	_replace_in_file(repo_path / "images" / "titlepage.svg", "TITLE_STRING", title_string)
+	_replace_in_file(repo_path / "images" / "cover.svg", "TITLE_STRING", title_string)
 
 	# Create the titlepage SVG
 	contributors = {}
@@ -506,17 +538,22 @@ def _create_draft(args: Namespace):
 	if args.illustrator:
 		contributors["illustrated by"] = args.illustrator
 
-	with open(repo_name / "images" / "titlepage.svg", "w", encoding="utf-8") as file:
+	with open(repo_path / "images" / "titlepage.svg", "w", encoding="utf-8") as file:
 		file.write(_generate_titlepage_svg(args.title, args.author, contributors, title_string))
 
 	# Create the cover SVG
-	with open(repo_name / "images" / "cover.svg", "w", encoding="utf-8") as file:
+	with open(repo_path / "images" / "cover.svg", "w", encoding="utf-8") as file:
 		file.write(_generate_cover_svg(args.title, args.author, title_string))
 
-	if args.pg_url:
-		se.replace_in_file(repo_name / "src" / "epub" / "text" / "imprint.xhtml", "PG_URL", args.pg_url)
+	# Build the cover/titlepage for distribution
+	epub = SeEpub(repo_path)
+	epub.generate_cover_svg()
+	epub.generate_titlepage_svg()
 
-	with open(repo_name / "src" / "epub" / "text" / "colophon.xhtml", "r+", encoding="utf-8") as file:
+	if args.pg_url:
+		_replace_in_file(repo_path / "src" / "epub" / "text" / "imprint.xhtml", "PG_URL", args.pg_url)
+
+	with open(repo_path / "src" / "epub" / "text" / "colophon.xhtml", "r+", encoding="utf-8") as file:
 		colophon_xhtml = file.read()
 
 		colophon_xhtml = colophon_xhtml.replace("SE_IDENTIFIER", identifier)
@@ -535,10 +572,12 @@ def _create_draft(args: Namespace):
 			if pg_producers:
 				producers_xhtml = ""
 				for i, producer in enumerate(pg_producers):
-					if "Distributed Proofreading" in producer:
+					if "Distributed Proofread" in producer:
 						producers_xhtml = producers_xhtml + "<a href=\"https://www.pgdp.net\">The Online Distributed Proofreading Team</a>"
+					elif "anonymous" in producer.lower():
+						producers_xhtml = producers_xhtml + "<b class=\"name\">An Anonymous Volunteer</b>"
 					else:
-						producers_xhtml = producers_xhtml + f"<b class=\"name\">{producer}</b>"
+						producers_xhtml = producers_xhtml + f"<b class=\"name\">{producer.strip('.')}</b>"
 
 					if i < len(pg_producers) - 1:
 						producers_xhtml = producers_xhtml + ", "
@@ -554,51 +593,51 @@ def _create_draft(args: Namespace):
 		file.write(colophon_xhtml)
 		file.truncate()
 
-	with open(repo_name / "src" / "epub" / "content.opf", "r+", encoding="utf-8") as file:
-		metadata_xhtml = file.read()
+	with open(repo_path / "src" / "epub" / "content.opf", "r+", encoding="utf-8") as file:
+		metadata_xml = file.read()
 
-		metadata_xhtml = metadata_xhtml.replace("SE_IDENTIFIER", identifier)
-		metadata_xhtml = metadata_xhtml.replace(">AUTHOR<", f">{args.author}<")
-		metadata_xhtml = metadata_xhtml.replace(">TITLE_SORT<", f">{sorted_title}<")
-		metadata_xhtml = metadata_xhtml.replace(">TITLE<", f">{args.title}<")
-		metadata_xhtml = metadata_xhtml.replace("VCS_IDENTIFIER", str(repo_name))
+		metadata_xml = metadata_xml.replace("SE_IDENTIFIER", identifier)
+		metadata_xml = metadata_xml.replace(">AUTHOR<", f">{args.author}<")
+		metadata_xml = metadata_xml.replace(">TITLE_SORT<", f">{sorted_title}<")
+		metadata_xml = metadata_xml.replace(">TITLE<", f">{args.title}<")
+		metadata_xml = metadata_xml.replace("VCS_IDENTIFIER", str(repo_name))
 
 		if pg_producers:
 			producers_xhtml = ""
 			i = 1
 			for producer in pg_producers:
-				producers_xhtml = producers_xhtml + f"\t\t<dc:contributor id=\"transcriber-{i}\">{producer}</dc:contributor>\n"
-
-				if "Distributed Proofreading" in producer:
-					producers_xhtml = producers_xhtml + f"\t\t<meta property=\"file-as\" refines=\"#transcriber-{i}\">Online Distributed Proofreading Team, The</meta>\n\t\t<meta property=\"se:url.homepage\" refines=\"#transcriber-{i}\">https://pgdp.net</meta>\n"
+				if "Distributed Proofread" in producer:
+					producers_xhtml = producers_xhtml + f"\t\t<dc:contributor id=\"transcriber-{i}\">The Online Distributed Proofreading Team</dc:contributor>\n\t\t<meta property=\"file-as\" refines=\"#transcriber-{i}\">Online Distributed Proofreading Team, The</meta>\n\t\t<meta property=\"se:url.homepage\" refines=\"#transcriber-{i}\">https://pgdp.net</meta>\n"
+				elif "anonymous" in producer.lower():
+					producers_xhtml = producers_xhtml + f"\t\t<dc:contributor id=\"transcriber-{i}\">An Anonymous Volunteer</dc:contributor>\n\t\t<meta property=\"file-as\" refines=\"#transcriber-{i}\">Anonymous Volunteer, An</meta>\n"
 				else:
-					producers_xhtml = producers_xhtml + f"\t\t<meta property=\"file-as\" refines=\"#transcriber-{i}\">TRANSCRIBER_SORT</meta>\n"
+					producers_xhtml = producers_xhtml + f"\t\t<dc:contributor id=\"transcriber-{i}\">{producer.strip('.')}</dc:contributor>\n\t\t<meta property=\"file-as\" refines=\"#transcriber-{i}\">TRANSCRIBER_SORT</meta>\n"
 
 				producers_xhtml = producers_xhtml + f"\t\t<meta property=\"role\" refines=\"#transcriber-{i}\" scheme=\"marc:relators\">trc</meta>\n"
 
 				i = i + 1
 
-			metadata_xhtml = regex.sub(r"\t\t<dc:contributor id=\"transcriber-1\">TRANSCRIBER</dc:contributor>\s*<meta property=\"file-as\" refines=\"#transcriber-1\">TRANSCRIBER_SORT</meta>\s*<meta property=\"se:url.homepage\" refines=\"#transcriber-1\">TRANSCRIBER_URL</meta>\s*<meta property=\"role\" refines=\"#transcriber-1\" scheme=\"marc:relators\">trc</meta>", "\t\t" + producers_xhtml.strip(), metadata_xhtml, flags=regex.DOTALL)
+			metadata_xml = regex.sub(r"\t\t<dc:contributor id=\"transcriber-1\">TRANSCRIBER</dc:contributor>\s*<meta property=\"file-as\" refines=\"#transcriber-1\">TRANSCRIBER_SORT</meta>\s*<meta property=\"se:url.homepage\" refines=\"#transcriber-1\">TRANSCRIBER_URL</meta>\s*<meta property=\"role\" refines=\"#transcriber-1\" scheme=\"marc:relators\">trc</meta>", "\t\t" + producers_xhtml.strip(), metadata_xml, flags=regex.DOTALL)
 
 		if author_wiki_url:
-			metadata_xhtml = metadata_xhtml.replace(">AUTHOR_WIKI_URL<", f">{author_wiki_url}<")
+			metadata_xml = metadata_xml.replace(">AUTHOR_WIKI_URL<", f">{author_wiki_url}<")
 
 		if author_nacoaf_url:
-			metadata_xhtml = metadata_xhtml.replace(">AUTHOR_NACOAF_URL<", f">{author_nacoaf_url}<")
+			metadata_xml = metadata_xml.replace(">AUTHOR_NACOAF_URL<", f">{author_nacoaf_url}<")
 
 		if ebook_wiki_url:
-			metadata_xhtml = metadata_xhtml.replace(">EBOOK_WIKI_URL<", f">{ebook_wiki_url}<")
+			metadata_xml = metadata_xml.replace(">EBOOK_WIKI_URL<", f">{ebook_wiki_url}<")
 
 		if args.translator:
-			metadata_xhtml = metadata_xhtml.replace(">TRANSLATOR<", f">{args.translator}<")
+			metadata_xml = metadata_xml.replace(">TRANSLATOR<", f">{args.translator}<")
 
 			if translator_wiki_url:
-				metadata_xhtml = metadata_xhtml.replace(">TRANSLATOR_WIKI_URL<", f">{translator_wiki_url}<")
+				metadata_xml = metadata_xml.replace(">TRANSLATOR_WIKI_URL<", f">{translator_wiki_url}<")
 
 			if translator_nacoaf_url:
-				metadata_xhtml = metadata_xhtml.replace(">TRANSLATOR_NACOAF_URL<", f">{translator_nacoaf_url}<")
+				metadata_xml = metadata_xml.replace(">TRANSLATOR_NACOAF_URL<", f">{translator_nacoaf_url}<")
 		else:
-			metadata_xhtml = regex.sub(r"<dc:contributor id=\"translator\">.+?<dc:contributor id=\"artist\">", "<dc:contributor id=\"artist\">", metadata_xhtml, flags=regex.DOTALL)
+			metadata_xml = regex.sub(r"<dc:contributor id=\"translator\">.+?<dc:contributor id=\"artist\">", "<dc:contributor id=\"artist\">", metadata_xml, flags=regex.DOTALL)
 
 		if args.pg_url:
 			if pg_subjects:
@@ -615,7 +654,7 @@ def _create_draft(args: Namespace):
 
 					# Now, get the LCSH ID by querying LCSH directly.
 					try:
-						response = requests.get(f"http://id.loc.gov/search/?q=%22{urllib.parse.quote(subject)}%22")
+						response = requests.get(f"https://id.loc.gov/search/?q=%22{urllib.parse.quote(subject)}%22")
 						result = regex.search(fr"<a title=\"Click to view record\" href=\"/authorities/subjects/([^\"]+?)\">{regex.escape(subject.replace(' -- ', '--'))}</a>", response.text)
 
 						loc_id = "Unknown"
@@ -627,21 +666,21 @@ def _create_draft(args: Namespace):
 						subject_xhtml = subject_xhtml + f"\t\t<meta property=\"term\" refines=\"#subject-{i}\">{loc_id}</meta>\n"
 
 					except Exception as ex:
-						raise se.RemoteCommandErrorException(f"Couldn’t connect to `id.loc.gov`. Error: {ex}")
+						raise se.RemoteCommandErrorException(f"Couldn’t connect to [url][link=https://id.loc.gov]https://id.loc.gov[/][/]. Exception: {ex}")
 
 					i = i + 1
 
-				metadata_xhtml = regex.sub(r"\t\t<dc:subject id=\"subject-1\">SUBJECT_1</dc:subject>\s*<dc:subject id=\"subject-2\">SUBJECT_2</dc:subject>\s*<meta property=\"authority\" refines=\"#subject-1\">LCSH</meta>\s*<meta property=\"term\" refines=\"#subject-1\">LCSH_ID_1</meta>\s*<meta property=\"authority\" refines=\"#subject-2\">LCSH</meta>\s*<meta property=\"term\" refines=\"#subject-2\">LCSH_ID_2</meta>", "\t\t" + subject_xhtml.strip(), metadata_xhtml)
+				metadata_xml = regex.sub(r"\t\t<dc:subject id=\"subject-1\">SUBJECT_1</dc:subject>\s*<dc:subject id=\"subject-2\">SUBJECT_2</dc:subject>\s*<meta property=\"authority\" refines=\"#subject-1\">LCSH</meta>\s*<meta property=\"term\" refines=\"#subject-1\">LCSH_ID_1</meta>\s*<meta property=\"authority\" refines=\"#subject-2\">LCSH</meta>\s*<meta property=\"term\" refines=\"#subject-2\">LCSH_ID_2</meta>", "\t\t" + subject_xhtml.strip(), metadata_xml)
 
-			metadata_xhtml = metadata_xhtml.replace("<dc:language>LANG</dc:language>", f"<dc:language>{pg_language}</dc:language>")
-			metadata_xhtml = metadata_xhtml.replace("<dc:source>PG_URL</dc:source>", f"<dc:source>{args.pg_url}</dc:source>")
+			metadata_xml = metadata_xml.replace("<dc:language>LANG</dc:language>", f"<dc:language>{pg_language}</dc:language>")
+			metadata_xml = metadata_xml.replace("<dc:source>PG_URL</dc:source>", f"<dc:source>{args.pg_url}</dc:source>")
 
 		file.seek(0)
-		file.write(metadata_xhtml)
+		file.write(metadata_xml)
 		file.truncate()
 
 	# Set up local git repo
-	repo = git.Repo.init(repo_name)
+	repo = git.Repo.init(repo_path)
 
 	if args.email:
 		with repo.config_writer() as config:
@@ -666,8 +705,8 @@ def create_draft() -> int:
 	args = parser.parse_args()
 
 	if args.pg_url and not regex.match("^https?://www.gutenberg.org/ebooks/[0-9]+$", args.pg_url):
-		se.print_error("Project Gutenberg URL must look like: `https://www.gutenberg.org/ebooks/<EBOOK-ID>`")
-		return se.InvalidInputException.code
+		se.print_error("Project Gutenberg URL must look like: [url]https://www.gutenberg.org/ebooks/<EBOOK-ID>[url].")
+		return se.InvalidArgumentsException.code
 
 	try:
 		_create_draft(args)
