@@ -10,7 +10,7 @@ import struct
 from html import unescape
 from typing import List, Callable, Dict
 import regex
-from PIL import Image, ImageMath, PngImagePlugin
+from PIL import Image, ImageMath, PngImagePlugin, UnidentifiedImageError
 import importlib_resources
 from lxml import etree
 
@@ -154,7 +154,7 @@ def remove_image_metadata(filename: Path) -> None:
 			jpeg_data = file.read()
 
 			if jpeg_data[0:2] != b"\xff\xd8":
-				raise se.InvalidFileException(f"Invalid JPEG file: `{filename}`")
+				raise se.InvalidFileException(f"Invalid JPEG file: [path][link=file://{filename.resolve()}]{filename}[/].")
 
 			exif_segments = []
 			head = 2
@@ -170,7 +170,7 @@ def remove_image_metadata(filename: Path) -> None:
 				head = end_point
 
 				if head >= len(jpeg_data):
-					raise se.InvalidFileException(f"Invalid JPEG file: `{filename}`")
+					raise se.InvalidFileException(f"Invalid JPEG file: [path][link=file://{filename.resolve()}]{filename}[/].")
 
 				# See https://www.disktuna.com/list-of-jpeg-markers/
 				# and https://exiftool.org/TagNames/JPEG.html
@@ -193,7 +193,11 @@ def remove_image_metadata(filename: Path) -> None:
 			file.truncate()
 	else:
 		# PNG and other image types we expect are lossless so we can use PIL to remove metadata
-		image = Image.open(filename)
+		try:
+			image = Image.open(filename)
+		except UnidentifiedImageError:
+			raise se.InvalidFileException(f"Couldn’t identify image type of [path][link=file://{filename.resolve()}]{filename}[/].")
+
 		data = list(image.getdata())
 
 		image_without_exif = Image.new(image.mode, image.size)
@@ -222,7 +226,7 @@ def remove_image_metadata(filename: Path) -> None:
 			# the save driver throws an exception
 			image_without_exif.save(str(filename), compression="tiff_adobe_deflate")
 		else:
-			image_without_exif.save(filename)
+			image_without_exif.save(str(filename))
 
 def svg_text_to_paths(in_svg: Path, out_svg: Path, remove_style=True) -> None:
 	"""
@@ -250,7 +254,11 @@ def svg_text_to_paths(in_svg: Path, out_svg: Path, remove_style=True) -> None:
 		font = _parse_font(font_path)
 		fonts.append(font)
 	svg_in_raw = open(in_svg, "rt").read()
-	xml = etree.fromstring(str.encode(svg_in_raw))
+
+	try:
+		xml = etree.fromstring(str.encode(svg_in_raw))
+	except:
+		raise se.InvalidXmlException(f"Couldn’t parse SVG file: [path][link={in_svg.resolve()}]{in_svg}[/][/]")
 
 	svg_ns = "{http://www.w3.org/2000/svg}"
 
@@ -272,7 +280,7 @@ def svg_text_to_paths(in_svg: Path, out_svg: Path, remove_style=True) -> None:
 			text = elem.text
 
 			if not text:
-				raise se.InvalidFileException(f"SVG `<text>` element has no content. File: `{in_svg}`")
+				raise se.InvalidFileException(f"SVG [xml]<text>[/] element has no content. File: [path][link=file://{in_svg.resolve()}]{in_svg}[/].")
 
 			elem.tag = "g"
 			# Replace <text> tag with <g> tag
@@ -288,7 +296,7 @@ def svg_text_to_paths(in_svg: Path, out_svg: Path, remove_style=True) -> None:
 
 	xmlstr = etree.tostring(xml, pretty_print=True).decode("UTF-8")
 	result_all_text = xmlstr.replace("ns0:", "").replace(":ns0", "")
-	result_all_text = se.formatting.format_xhtml(result_all_text)
+	result_all_text = se.formatting.format_xml(result_all_text)
 	open(out_svg, "wt").write(result_all_text)
 
 def _apply_css(elem: etree.Element, css_text: str) -> dict:
@@ -366,7 +374,7 @@ def _add_svg_paths_to_group(g_elem: etree.Element, text_properties: Dict) -> Non
 	# Required properties to make any progress
 	for key in "x y font text font-size".split():
 		if not key in text_properties:
-			raise se.InvalidCssException(f"svg_text_to_paths: Missing key `{key}` in `text_properties` for `<{g_elem.tag}>` element in `./images/titlepage.svg` or `./images/cover.svg`.")
+			raise se.InvalidCssException(f"svg_text_to_paths: Missing key [text]{key}[/] in [text]text_properties[/] for [xml]<{g_elem.tag}>[/] element in [path]./images/titlepage.svg[/] or [path]./images/cover.svg[/].")
 	# We know we have x, y, text, font-size, and font so we can render vectors.
 	# Now set up some defaults if not specified.
 	text_properties["font-size"] = float(text_properties["font-size"].replace("px", "")) # NOTE assumes pixels and ignores it
