@@ -10,12 +10,14 @@ import datetime
 import fnmatch
 import html
 import os
-import tempfile
 from pathlib import Path
-from typing import List, Optional
-import git
-import regex
+from typing import List, Optional, Tuple, Union
+
 from bs4 import BeautifulSoup, Tag
+import git
+from natsort import natsorted
+import regex
+
 import se
 import se.easy_xml
 import se.formatting
@@ -82,33 +84,48 @@ class SeEpub:
 
 	path = Path()
 	metadata_file_path = Path()
-	metadata_xhtml = ""
+	metadata_xml = ""
 	local_css = ""
-	__metadata_tree = None
+	_metadata_dom = None
 	_generated_identifier = None
 	_generated_github_repo_url = None
+	_repo = None # git.Repo object
 	_last_commit = None # GitCommit object
 	__endnotes_soup = None # bs4 soup object of the endnotes.xhtml file
 	_endnotes: Optional[List[Endnote]] = None # List of Endnote objects
 
-	def __init__(self, epub_root_directory: str):
+	def __init__(self, epub_root_directory: Union[str, Path]):
 		try:
 			self.path = Path(epub_root_directory).resolve()
 
 			if not self.path.is_dir():
-				raise se.InvalidSeEbookException(f"Not a directory: `{self.path}`")
+				raise se.InvalidSeEbookException(f"Not a directory: [path][link=file://{self.path}]{self.path}[/][/].")
 
 			with open(self.path / "src" / "META-INF" / "container.xml", "r", encoding="utf-8") as file:
 				container_tree = se.easy_xml.EasyXmlTree(file.read())
-				self.metadata_file_path = self.path / "src" / container_tree.xpath("//container:container/container:rootfiles/container:rootfile[@media-type=\"application/oebps-package+xml\"]/@full-path")[0]
+				self.metadata_file_path = self.path / "src" / container_tree.xpath("/container:container/container:rootfiles/container:rootfile[@media-type=\"application/oebps-package+xml\"]/@full-path")[0]
 
 			with open(self.metadata_file_path, "r", encoding="utf-8") as file:
-				self.metadata_xhtml = file.read()
+				self.metadata_xml = file.read()
 
-			if "<dc:identifier id=\"uid\">url:https://standardebooks.org/ebooks/" not in self.metadata_xhtml:
+			if "<dc:identifier id=\"uid\">url:https://standardebooks.org/ebooks/" not in self.metadata_xml:
 				raise se.InvalidSeEbookException
 		except:
-			raise se.InvalidSeEbookException(f"Not a Standard Ebooks source directory: `{self.path}`")
+			raise se.InvalidSeEbookException(f"Not a Standard Ebooks source directory: [path][link=file://{self.path}]{self.path}[/][/].")
+
+	@property
+	def repo(self) -> git.Repo:
+		"""
+		Accessor
+		"""
+
+		if not self._repo:
+			try:
+				self._repo = git.Repo(self.path)
+			except:
+				raise se.InvalidSeEbookException("Couldn’t access this ebook’s Git repository.")
+
+		return self._repo
 
 	@property
 	def last_commit(self) -> Optional[GitCommit]:
@@ -146,15 +163,15 @@ class SeEpub:
 			# Add authors
 			identifier = "url:https://standardebooks.org/ebooks/"
 			authors = []
-			for author in self._metadata_tree.xpath("//dc:creator"):
-				authors.append(author.inner_html())
-				identifier += se.formatting.make_url_safe(author.inner_html()) + "_"
+			for author in self.metadata_dom.xpath("/package/metadata/dc:creator"):
+				authors.append(author.text)
+				identifier += se.formatting.make_url_safe(author.text) + "_"
 
 			identifier = identifier.strip("_") + "/"
 
 			# Add title
-			for title in self._metadata_tree.xpath("//dc:title[@id=\"title\"]"):
-				identifier += se.formatting.make_url_safe(title.inner_html()) + "/"
+			for title in self.metadata_dom.xpath("/package/metadata/dc:title[@id=\"title\"]"):
+				identifier += se.formatting.make_url_safe(title.text) + "/"
 
 			# For contributors, we add both translators and illustrators.
 			# However, we may not include specific translators or illustrators in certain cases, namely
@@ -165,25 +182,25 @@ class SeEpub:
 			illustrators = []
 			translators_have_display_seq = False
 			illustrators_have_display_seq = False
-			for role in self._metadata_tree.xpath("//opf:meta[@property=\"role\"]"):
+			for role in self.metadata_dom.xpath("/package/metadata/meta[@property=\"role\"]"):
 				contributor_id = role.attribute("refines").lstrip("#")
-				contributor_element = self._metadata_tree.xpath("//dc:contributor[@id=\"" + contributor_id + "\"]")
+				contributor_element = self.metadata_dom.xpath("/package/metadata/dc:contributor[@id=\"" + contributor_id + "\"]")
 				if contributor_element:
-					contributor = {"name": contributor_element[0].inner_html(), "include": True, "display_seq": None}
-					display_seq = self._metadata_tree.xpath("//opf:meta[@property=\"display-seq\"][@refines=\"#" + contributor_id + "\"]")
+					contributor = {"name": contributor_element[0].text, "include": True, "display_seq": None}
+					display_seq = self.metadata_dom.xpath("/package/metadata/meta[@property=\"display-seq\"][@refines=\"#" + contributor_id + "\"]")
 
-					if display_seq and int(display_seq[0].inner_html()) == 0:
+					if display_seq and int(display_seq[0].text) == 0:
 						contributor["include"] = False
 						display_seq = []
 
-					if role.inner_html() == "trl":
+					if role.text == "trl":
 						if display_seq:
 							contributor["display_seq"] = display_seq[0]
 							translators_have_display_seq = True
 
 						translators.append(contributor)
 
-					if role.inner_html() == "ill":
+					if role.text == "ill":
 						if display_seq:
 							contributor["display_seq"] = display_seq[0]
 							illustrators_have_display_seq = True
@@ -246,7 +263,7 @@ class SeEpub:
 				with open(self.path / "src" / "epub" / "text" / "endnotes.xhtml") as file:
 					self.__endnotes_soup = BeautifulSoup(file.read(), "html.parser")
 			except:
-				raise se.InvalidFileException(f"Could't open file: `{str(self.path / 'src' / 'epub' / 'text' / 'endnotes.xhtml')}`")
+				raise se.InvalidFileException(f"Could't open file: [path][link=file://{self.path / 'src' / 'epub' / 'text' / 'endnotes.xhtml'}]{self.path / 'src' / 'epub' / 'text' / 'endnotes.xhtml'}[/][/].")
 
 		return self.__endnotes_soup
 
@@ -290,18 +307,18 @@ class SeEpub:
 		return self._endnotes
 
 	@property
-	def _metadata_tree(self) -> se.easy_xml.EasyXmlTree:
+	def metadata_dom(self) -> se.easy_xml.EasyXmlTree:
 		"""
 		Accessor
 		"""
 
-		if self.__metadata_tree is None:
+		if self._metadata_dom is None:
 			try:
-				self.__metadata_tree = se.easy_xml.EasyXmlTree(self.metadata_xhtml)
+				self._metadata_dom = se.easy_xml.EasyOpfTree(self.metadata_xml)
 			except Exception as ex:
-				raise se.InvalidSeEbookException(f"Couldn’t parse `{self.metadata_file_path}`: {ex}")
+				raise se.InvalidXmlException(f"Couldn’t parse [path][link=file://{self.metadata_file_path}]{self.metadata_file_path}[/][/]. Exception: {ex}")
 
-		return self.__metadata_tree
+		return self._metadata_dom
 
 	@staticmethod
 	def _new_bs4_tag(section: Tag, output_soup: BeautifulSoup) -> Tag:
@@ -338,7 +355,7 @@ class SeEpub:
 
 		# Quick sanity check before we begin
 		if "id" not in section.attrs or (section.parent.name.lower() != "body" and "id" not in section.parent.attrs):
-			raise se.InvalidXhtmlException("Section without `id` attribute.")
+			raise se.InvalidXhtmlException("Section without [attr]id[/] attribute.")
 
 		# Try to find our parent tag in the output, by ID.
 		# If it's not in the output, then append it to the tag's closest parent by ID (or <body>), then iterate over its children and do the same.
@@ -359,12 +376,12 @@ class SeEpub:
 				else:
 					existing_section[0].append(child)
 
-	def recompose(self) -> str:
+	def recompose(self, output_xhtml5: bool) -> str:
 		"""
-		Iterate over the XHTML files in this epub and "recompose" them into a single HTML5 string representing this ebook.
+		Iterate over the XHTML files in this epub and "recompose" them into a single XHTML string representing this ebook.
 
 		INPUTS
-		None
+		output_xhtml5: true to output XHTML5 instead of HTML5
 
 		OUTPUTS
 		A string of HTML5 representing the entire recomposed ebook.
@@ -378,11 +395,22 @@ class SeEpub:
 		title = html.escape(metadata_soup.find("dc:title").contents[0])
 		css = ""
 		with open(self.path / "src" / "epub" / "css" / "core.css", "r", encoding="utf-8") as file:
-			css = regex.sub(r"@.+?;", "", file.read()).strip()
+			css = file.read()
 
 		with open(self.path / "src" / "epub" / "css" / "local.css", "r", encoding="utf-8") as file:
-			css = css + "\n\n\n/* local.css */" + regex.sub(r"@.+?;", "", file.read())
-			css = "\t\t\t".join(css.splitlines(True))
+			css = css + "\n\n\n/* local.css */" + file.read()
+
+		namespaces = set(regex.findall(r"@namespace.+?;", css))
+
+		css = regex.sub(r"@(charset|namespace).+?;", "", css).strip()
+
+		if namespaces:
+			css = "\n" + css
+
+		for namespace in namespaces:
+			css = namespace + "\n" + css
+
+		css = "\t\t\t".join(css.splitlines(True))
 
 		output_xhtml = "<?xml version=\"1.0\" encoding=\"utf-8\"?><html xmlns=\"http://www.w3.org/1999/xhtml\" xmlns:epub=\"http://www.idpf.org/2007/ops\" epub:prefix=\"z3998: http://www.daisy.org/z3998/2012/vocab/structure/, se: https://standardebooks.org/vocab/1.0\"><head><meta charset=\"utf-8\"/><title>" + title + "</title><style/></head><body></body></html>"
 		output_soup = BeautifulSoup(output_xhtml, "lxml")
@@ -417,36 +445,28 @@ class SeEpub:
 
 				output_xhtml = regex.sub(fr"<img.+?src=\"\.\./images/{match}\.svg\".*?/>", svg, output_xhtml)
 
-		with tempfile.NamedTemporaryFile(mode="w+", delete=False) as temp_file:
-			temp_file.write(output_xhtml)
-			file_name = Path(temp_file.name)
-			file_name_xhtml = Path(str(file_name) + ".xhtml")
-
-		file_name.rename(file_name_xhtml)
-
 		# All done, clean the output
-		se.formatting.format_xhtml_file(file_name_xhtml, False, False, file_name_xhtml.name == "endnotes.xhtml", file_name_xhtml.name == "colophon.xhtml")
+		output_xhtml = se.formatting.format_xhtml(output_xhtml)
 
-		with open(file_name_xhtml, "r", encoding="utf-8") as file:
-			xhtml = file.read()
+		# Insert our CSS. We do this after `clean` because `clean` will escape > in the CSS
+		output_xhtml = regex.sub(r"<style/>", "<style>\n\t\t\t" + css + "\t\t</style>", output_xhtml)
 
+		if output_xhtml5:
+			output_xhtml = output_xhtml.replace("\t\t<meta charset=\"utf-8\"/>\n", "")
+			output_xhtml = output_xhtml.replace("\t\t<style/>\n", "")
+		else:
 			# Remove xml declaration and re-add the doctype
-			xhtml = regex.sub(r"<\?xml.+?\?>", "<!doctype html>", xhtml)
-			xhtml = regex.sub(r" epub:prefix=\".+?\"", "", xhtml)
-
-			# Insert our CSS. We do this after `clean` because `clean` will escape > in the CSS
-			xhtml = regex.sub(r"<style/>", "<style>\n\t\t\t" + css + "\t\t</style>", xhtml)
+			output_xhtml = regex.sub(r"<\?xml.+?\?>", "<!doctype html>", output_xhtml)
+			output_xhtml = regex.sub(r" epub:prefix=\".+?\"", "", output_xhtml)
 
 			# Make some replacements for HTML5 compatibility
-			xhtml = xhtml.replace("epub:type", "data-epub-type")
-			xhtml = xhtml.replace("epub|type", "data-epub-type")
-			xhtml = xhtml.replace("xml:lang", "lang")
-			xhtml = xhtml.replace("<html", f"<html lang=\"{metadata_soup.find('dc:language').string}\"")
-			xhtml = regex.sub(" xmlns.+?=\".+?\"", "", xhtml)
+			output_xhtml = output_xhtml.replace("epub:type", "data-epub-type")
+			output_xhtml = output_xhtml.replace("epub|type", "data-epub-type")
+			output_xhtml = output_xhtml.replace("xml:lang", "lang")
+			output_xhtml = output_xhtml.replace("<html", f"<html lang=\"{metadata_soup.find('dc:language').string}\"")
+			output_xhtml = regex.sub(" xmlns.+?=\".+?\"", "", output_xhtml)
 
-		file_name_xhtml.unlink()
-
-		return xhtml
+		return output_xhtml
 
 	def generate_titlepage_svg(self) -> None:
 		"""
@@ -460,7 +480,7 @@ class SeEpub:
 		"""
 		source_images_directory = self.path / "images"
 		source_titlepage_svg_filename = source_images_directory / "titlepage.svg"
-		dest_images_directory = self.path / "src" / "epub" / "images"
+		dest_images_directory = self.path / "src/epub/images"
 		dest_titlepage_svg_filename = dest_images_directory / "titlepage.svg"
 
 		if source_titlepage_svg_filename.is_file():
@@ -481,7 +501,7 @@ class SeEpub:
 		source_images_directory = self.path / "images"
 		source_cover_jpg_filename = source_images_directory / "cover.jpg"
 		source_cover_svg_filename = source_images_directory / "cover.svg"
-		dest_images_directory = self.path / "src" / "epub" / "images"
+		dest_images_directory = self.path / "src/epub/images"
 		dest_cover_svg_filename = dest_images_directory / "cover.svg"
 
 		# Create output directory if it doesn't exist
@@ -530,7 +550,7 @@ class SeEpub:
 		source_directory = self.path / "src"
 
 		try:
-			endnotes_filename = source_directory / "epub" / "text" / "endnotes.xhtml"
+			endnotes_filename = source_directory / "epub/text/endnotes.xhtml"
 			with open(endnotes_filename, "r+", encoding="utf-8") as file:
 				xhtml = file.read()
 				soup = BeautifulSoup(xhtml, "lxml")
@@ -561,7 +581,7 @@ class SeEpub:
 				file.truncate()
 
 		except Exception:
-			raise se.InvalidSeEbookException(f"Couldn’t open endnotes file: `{endnotes_filename}`")
+			raise se.InvalidSeEbookException(f"Couldn’t open endnotes file: [path][link=file://{endnotes_filename}]{endnotes_filename}[/][/].")
 
 		with concurrent.futures.ProcessPoolExecutor() as executor:
 			for root, _, filenames in os.walk(source_directory):
@@ -577,22 +597,22 @@ class SeEpub:
 		If this ebook has not yet been released, set the first release timestamp in the metadata file.
 		"""
 
-		if "<dc:date>1900-01-01T00:00:00Z</dc:date>" in self.metadata_xhtml:
+		if "<dc:date>1900-01-01T00:00:00Z</dc:date>" in self.metadata_xml:
 			now = datetime.datetime.utcnow()
 			now_iso = regex.sub(r"\.[0-9]+$", "", now.isoformat()) + "Z"
 			now_iso = regex.sub(r"\+.+?Z$", "Z", now_iso)
 			now_friendly = f"{now:%B %e, %Y, %l:%M <abbr class=\"time eoc\">%p</abbr>}"
 			now_friendly = regex.sub(r"\s+", " ", now_friendly).replace("AM", "a.m.").replace("PM", "p.m.").replace(" <abbr", " <abbr")
 
-			self.metadata_xhtml = regex.sub(r"<dc:date>[^<]+?</dc:date>", f"<dc:date>{now_iso}</dc:date>", self.metadata_xhtml)
-			self.metadata_xhtml = regex.sub(r"<meta property=\"dcterms:modified\">[^<]+?</meta>", f"<meta property=\"dcterms:modified\">{now_iso}</meta>", self.metadata_xhtml)
+			self.metadata_xml = regex.sub(r"<dc:date>[^<]+?</dc:date>", f"<dc:date>{now_iso}</dc:date>", self.metadata_xml)
+			self.metadata_xml = regex.sub(r"<meta property=\"dcterms:modified\">[^<]+?</meta>", f"<meta property=\"dcterms:modified\">{now_iso}</meta>", self.metadata_xml)
 
 			with open(self.metadata_file_path, "w", encoding="utf-8") as file:
 				file.seek(0)
-				file.write(self.metadata_xhtml)
+				file.write(self.metadata_xml)
 				file.truncate()
 
-			self.__metadata_tree = None
+			self._metadata_dom = None
 
 			with open(self.path / "src" / "epub" / "text" / "colophon.xhtml", "r+", encoding="utf-8") as file:
 				xhtml = file.read()
@@ -620,11 +640,11 @@ class SeEpub:
 			with open(filename, "r", encoding="utf-8") as file:
 				text += " " + file.read()
 
-		self.metadata_xhtml = regex.sub(r"<meta property=\"se:reading-ease\.flesch\">[^<]*</meta>", f"<meta property=\"se:reading-ease.flesch\">{se.formatting.get_flesch_reading_ease(text)}</meta>", self.metadata_xhtml)
+		self.metadata_xml = regex.sub(r"<meta property=\"se:reading-ease\.flesch\">[^<]*</meta>", f"<meta property=\"se:reading-ease.flesch\">{se.formatting.get_flesch_reading_ease(text)}</meta>", self.metadata_xml)
 
 		with open(self.metadata_file_path, "w", encoding="utf-8") as file:
 			file.seek(0)
-			file.write(self.metadata_xhtml)
+			file.write(self.metadata_xml)
 			file.truncate()
 
 	def update_word_count(self) -> None:
@@ -648,11 +668,11 @@ class SeEpub:
 			with open(filename, "r", encoding="utf-8") as file:
 				word_count += se.formatting.get_word_count(file.read())
 
-		self.metadata_xhtml = regex.sub(r"<meta property=\"se:word-count\">[^<]*</meta>", f"<meta property=\"se:word-count\">{word_count}</meta>", self.metadata_xhtml)
+		self.metadata_xml = regex.sub(r"<meta property=\"se:word-count\">[^<]*</meta>", f"<meta property=\"se:word-count\">{word_count}</meta>", self.metadata_xml)
 
 		with open(self.metadata_file_path, "r+", encoding="utf-8") as file:
 			file.seek(0)
-			file.write(self.metadata_xhtml)
+			file.write(self.metadata_xml)
 			file.truncate()
 
 	def generate_manifest(self) -> str:
@@ -718,7 +738,7 @@ class SeEpub:
 
 				manifest.append(f"<item href=\"text/{filename}\" id=\"{filename}\" media-type=\"application/xhtml+xml\"{properties}/>")
 
-		manifest = se.natural_sort(manifest)
+		manifest = natsorted(manifest)
 
 		manifest_xhtml = "<manifest>\n\t<item href=\"toc.xhtml\" id=\"toc.xhtml\" media-type=\"application/xhtml+xml\" properties=\"nav\"/>\n"
 
@@ -743,7 +763,7 @@ class SeEpub:
 		excluded_files = se.IGNORED_FILENAMES + ["dedication.xhtml", "introduction.xhtml", "foreword.xhtml", "preface.xhtml", "epigraph.xhtml", "afterword.xhtml", "endnotes.xhtml"]
 		spine = ["<itemref idref=\"titlepage.xhtml\"/>", "<itemref idref=\"imprint.xhtml\"/>"]
 
-		filenames = se.natural_sort(os.listdir(self.path / "src" / "epub" / "text"))
+		filenames = natsorted(os.listdir(self.path / "src" / "epub" / "text"))
 
 		if "dedication.xhtml" in filenames:
 			spine.append("<itemref idref=\"dedication.xhtml\"/>")
@@ -776,8 +796,11 @@ class SeEpub:
 		if "loi.xhtml" in filenames:
 			spine.append("<itemref idref=\"loi.xhtml\"/>")
 
-		spine.append("<itemref idref=\"colophon.xhtml\"/>")
-		spine.append("<itemref idref=\"uncopyright.xhtml\"/>")
+		if "colophon.xhtml" in filenames:
+			spine.append("<itemref idref=\"colophon.xhtml\"/>")
+
+		if "uncopyright.xhtml" in filenames:
+			spine.append("<itemref idref=\"uncopyright.xhtml\"/>")
 
 		spine_xhtml = "<spine>\n"
 		for line in spine:
@@ -799,7 +822,7 @@ class SeEpub:
 		list of content files in the order given in the spine in content.opf
 		"""
 
-		return regex.findall(r"<itemref idref=\"(.*?)\"/>", self.metadata_xhtml)
+		return regex.findall(r"<itemref idref=\"(.*?)\"/>", self.metadata_xml)
 
 
 	def get_work_type(self) -> str:
@@ -814,7 +837,7 @@ class SeEpub:
 		"""
 
 		worktype = "fiction"  # default
-		subjects = regex.findall(r"<meta property=\"se:subject\">([^<]+?)</meta>", self.metadata_xhtml)
+		subjects = regex.findall(r"<meta property=\"se:subject\">([^<]+?)</meta>", self.metadata_xml)
 		if not subjects:
 			return worktype
 
@@ -843,7 +866,7 @@ class SeEpub:
 		Either the title of the book or the default WORKING_TITLE
 		"""
 
-		match = regex.search(r"<dc:title(?:.*?)>(.*?)</dc:title>", self.metadata_xhtml)
+		match = regex.search(r"<dc:title(?:.*?)>(.*?)</dc:title>", self.metadata_xml)
 		if match:
 			dc_title = match.group(1)
 		else:
@@ -859,9 +882,9 @@ class SeEpub:
 
 		from se.se_epub_lint import lint # pylint: disable=import-outside-toplevel
 
-		return lint(self, self.metadata_xhtml, skip_lint_ignore)
+		return lint(self, skip_lint_ignore)
 
-	def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, output_directory: Path, proof: bool, build_covers: bool, verbose: bool):
+	def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, output_directory: Path, proof: bool, build_covers: bool) -> None:
 		"""
 		The build() function is very big so for readability and maintainability
 		it's broken out to a separate file. Strictly speaking that file can be inlined
@@ -870,7 +893,7 @@ class SeEpub:
 
 		from se.se_epub_build import build # pylint: disable=import-outside-toplevel
 
-		build(self, self.metadata_xhtml, self._metadata_tree, run_epubcheck, build_kobo, build_kindle, output_directory, proof, build_covers, verbose)
+		build(self, run_epubcheck, build_kobo, build_kindle, output_directory, proof, build_covers)
 
 	def generate_toc(self) -> str:
 		"""
@@ -883,14 +906,15 @@ class SeEpub:
 
 		return generate_toc(self)
 
-	def generate_endnotes(self) -> str:
+	def generate_endnotes(self) -> Tuple[int, int]:
 		"""
 		Read the epub spine to regenerate all endnotes in order of appearance, starting from 1.
 		Changes are written to disk.
+
+		Returns a tuple of (found_endnote_count, changed_endnote_count)
 		"""
 
 		processed = 0
-		report = ""
 		current_note_number = 1
 		notes_changed = 0
 		change_list = []
@@ -901,12 +925,12 @@ class SeEpub:
 
 			processed += 1
 
-			file_path = self.path / "src" / "epub" / "text" / file_name
+			file_path = self.path / "src/epub/text" / file_name
 			try:
 				with open(file_path) as file:
 					soup = BeautifulSoup(file.read(), "lxml")
 			except:
-				raise se.InvalidFileException(f"Couldn’t open file: `{file_path}`")
+				raise se.InvalidFileException(f"Couldn’t open file: [path][link=file://{file_path}]{file_path}[/][/].")
 
 			links = soup.find_all("a")
 			needs_rewrite = False
@@ -936,9 +960,9 @@ class SeEpub:
 					match_old = lambda x, old=old_anchor: x.anchor == old
 					matches = list(filter(match_old, self.endnotes))
 					if not matches:
-						raise se.InvalidInputException(f"Couldn’t find endnote with anchor `{old_anchor}`")
+						raise se.InvalidInputException(f"Couldn’t find endnote with anchor [attr]{old_anchor}[/].")
 					if len(matches) > 1:
-						raise se.InvalidInputException(f"Duplicate anchors in endnotes file for anchor `{old_anchor}`")
+						raise se.InvalidInputException(f"Duplicate anchors in endnotes file for anchor [attr]{old_anchor}[/].")
 					# Found a single match, which is what we want
 					endnote = matches[0]
 					endnote.number = current_note_number
@@ -954,38 +978,33 @@ class SeEpub:
 				new_file.close()
 
 		if processed == 0:
-			report += "No files processed. Did you update the manifest and order the spine?\n"
-		else:
-			report += f"Found {current_note_number - 1} endnotes.\n"
-			if notes_changed > 0:
-				# Now we need to recreate the endnotes file
-				ol_tag = self._endnotes_soup.ol
-				ol_tag.clear()
+			raise se.InvalidInputException("No files processed. Did you update the manifest and order the spine?")
 
-				self.endnotes.sort(key=lambda endnote: endnote.number)
+		if notes_changed > 0:
+			# Now we need to recreate the endnotes file
+			ol_tag = self._endnotes_soup.ol
+			ol_tag.clear()
 
-				for endnote in self.endnotes:
-					if endnote.matched:
-						li_tag = self._endnotes_soup.new_tag("li")
-						li_tag["id"] = "note-" + str(endnote.number)
-						li_tag["epub:type"] = "endnote"
-						for content in endnote.contents:
-							if isinstance(content, Tag):
-								links = content.find_all("a")
-								for link in links:
-									epub_type = link.get("epub:type") or ""
-									if epub_type == "backlink":
-										href = link.get("href") or ""
-										if href:
-											link["href"] = endnote.source_file + "#noteref-" + str(endnote.number)
-							li_tag.append(content)
-						ol_tag.append(li_tag)
+			self.endnotes.sort(key=lambda endnote: endnote.number)
 
-				with open(self.path / "src" / "epub" / "text" / "endnotes.xhtml", "w") as file:
-					file.write(se.formatting.format_xhtml(str(self._endnotes_soup), is_endnotes_file=True))
+			for endnote in self.endnotes:
+				if endnote.matched:
+					li_tag = self._endnotes_soup.new_tag("li")
+					li_tag["id"] = "note-" + str(endnote.number)
+					li_tag["epub:type"] = "endnote"
+					for content in endnote.contents:
+						if isinstance(content, Tag):
+							links = content.find_all("a")
+							for link in links:
+								epub_type = link.get("epub:type") or ""
+								if epub_type == "backlink":
+									href = link.get("href") or ""
+									if href:
+										link["href"] = endnote.source_file + "#noteref-" + str(endnote.number)
+						li_tag.append(content)
+					ol_tag.append(li_tag)
 
-				report += "Changed {:d} endnote{}.".format(notes_changed, "s" if notes_changed != 1 else "")
-			else:
-				report += "No changes made."
+			with open(self.path / "src" / "epub" / "text" / "endnotes.xhtml", "w") as file:
+				file.write(se.formatting.format_xhtml(str(self._endnotes_soup)))
 
-		return report
+		return (current_note_number - 1, notes_changed)
