@@ -11,19 +11,19 @@ import os
 import shutil
 import subprocess
 import tempfile
-import sys
 from distutils.dir_util import copy_tree
 from hashlib import sha1
 from pathlib import Path
 from typing import List
 import importlib_resources
 
+from bs4 import BeautifulSoup
+from cairosvg import svg2png
+from natsort import natsorted
+from PIL import Image
 import lxml.cssselect
 import lxml.etree as etree
 import regex
-from bs4 import BeautifulSoup
-from cairosvg import svg2png
-from PIL import Image
 
 import se
 import se.easy_xml
@@ -42,8 +42,7 @@ COVER_THUMBNAIL_HEIGHT = int(COVER_SVG_HEIGHT / 4) # Cast to int required for PI
 SVG_OUTER_STROKE_WIDTH = 2
 SVG_TITLEPAGE_OUTER_STROKE_WIDTH = 4
 
-
-def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, output_directory: Path, proof: bool, build_covers: bool, verbose: bool) -> None:
+def build(self, run_epubcheck: bool, build_kobo: bool, build_kindle: bool, output_directory: Path, proof: bool, build_covers: bool) -> None:
 	"""
 	Entry point for `se build`
 	"""
@@ -57,22 +56,21 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 			# Look for default Mac calibre app path if none found in path
 			ebook_convert_path = Path("/Applications/calibre.app/Contents/MacOS/ebook-convert")
 			if not ebook_convert_path.exists():
-				raise se.MissingDependencyException("Couldn’t locate `ebook-convert`. Is `calibre` installed?")
+				raise se.MissingDependencyException("Couldn’t locate [bash]ebook-convert[/]. Is [bash]calibre[/] installed?")
 
 	if run_epubcheck:
 		if not shutil.which("java"):
-			raise se.MissingDependencyException("Couldn’t locate `java`. Is it installed?")
+			raise se.MissingDependencyException("Couldn’t locate [bash]java[/]. Is it installed?")
 
 	# Check the output directory and create it if it doesn't exist
 	try:
 		output_directory = output_directory.resolve()
 		output_directory.mkdir(parents=True, exist_ok=True)
 	except Exception:
-		raise se.FileExistsException(f"Couldn’t create output directory: `{output_directory}`")
+		raise se.FileExistsException(f"Couldn’t create output directory: [path][link=file://{output_directory}]{output_directory}[/][/].")
 
 	# All clear to start building!
-	if verbose:
-		print(f"Building {self.path} ...")
+	metadata_xml = self.metadata_xml
 
 	with tempfile.TemporaryDirectory() as temp_directory:
 		work_directory = Path(temp_directory)
@@ -85,22 +83,28 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 			pass
 
 		# By convention the ASIN is set to the SHA-1 sum of the book's identifying URL
-		identifier = metadata_tree.xpath("//dc:identifier")[0].inner_html().replace("url:", "")
-		asin = sha1(identifier.encode("utf-8")).hexdigest()
+		try:
+			identifier = self.metadata_dom.xpath("//dc:identifier")[0].inner_xml().replace("url:", "")
+			asin = sha1(identifier.encode("utf-8")).hexdigest()
+		except:
+			raise se.InvalidSeEbookException(f"Missing [xml]<dc:identifier>[/] element in [path][link=file://{self.metadata_file_path}]{self.metadata_file_path}[/][/].")
 
-		title = metadata_tree.xpath("//dc:title")[0].inner_html()
-		url_title = se.formatting.make_url_safe(title)
+		try:
+			title = self.metadata_dom.xpath("//dc:title")[0].inner_xml()
+			url_title = se.formatting.make_url_safe(title)
+		except:
+			raise se.InvalidSeEbookException(f"Missing [xml]<dc:title>[/] element in [path][link=file://{self.metadata_file_path}]{self.metadata_file_path}[/][/].")
 
 		url_author = ""
-		for author in metadata_tree.xpath("//dc:creator"):
-			url_author = url_author + se.formatting.make_url_safe(author.inner_html()) + "_"
+		for author in self.metadata_dom.xpath("//dc:creator"):
+			url_author = url_author + se.formatting.make_url_safe(author.inner_xml()) + "_"
 
 		url_author = url_author.rstrip("_")
 
-		epub_output_filename = "{}_{}{}.epub".format(url_author, url_title, ".proof" if proof else "")
-		epub3_output_filename = "{}_{}{}.epub3".format(url_author, url_title, ".proof" if proof else "")
-		kobo_output_filename = "{}_{}{}.kepub.epub".format(url_author, url_title, ".proof" if proof else "")
-		kindle_output_filename = "{}_{}{}.azw3".format(url_author, url_title, ".proof" if proof else "")
+		epub_output_filename = f"{url_author}_{url_title}{'.proof' if proof else ''}.epub"
+		epub3_output_filename = f"{url_author}_{url_title}{'.proof' if proof else ''}.epub3"
+		kobo_output_filename = f"{url_author}_{url_title}{'.proof' if proof else ''}.kepub.epub"
+		kindle_output_filename = f"{url_author}_{url_title}{'.proof' if proof else ''}.azw3"
 
 		# Clean up old output files if any
 		se.quiet_remove(output_directory / f"thumbnail_{asin}_EBOK_portrait.jpg")
@@ -127,11 +131,11 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 			last_updated_friendly = regex.sub(r"\s+", " ", last_updated_friendly).replace("AM", "a.m.").replace("PM", "p.m.").replace(" <abbr", " <abbr")
 
 			# Set modified date in content.opf
-			self.metadata_xhtml = regex.sub(r"<meta property=\"dcterms:modified\">[^<]+?</meta>", f"<meta property=\"dcterms:modified\">{last_updated_iso}</meta>", self.metadata_xhtml)
+			self.metadata_xml = regex.sub(r"<meta property=\"dcterms:modified\">[^<]+?</meta>", f"<meta property=\"dcterms:modified\">{last_updated_iso}</meta>", self.metadata_xml)
 
 			with open(work_epub_root_directory / "epub" / "content.opf", "w", encoding="utf-8") as file:
 				file.seek(0)
-				file.write(self.metadata_xhtml)
+				file.write(self.metadata_xml)
 				file.truncate()
 
 			# Update the colophon with release info
@@ -145,21 +149,8 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 				file.truncate()
 
 		# Output the pure epub3 file
-		if verbose:
-			print(f"\tBuilding {epub3_output_filename} ...", end="", flush=True)
-
 		se.epub.write_epub(work_epub_root_directory, output_directory / epub3_output_filename)
 
-		if verbose:
-			print(" OK")
-
-		if build_kobo:
-			if verbose:
-				print(f"\tBuilding {kobo_output_filename} ...", end="", flush=True)
-		else:
-			if verbose:
-				print(f"\tBuilding {epub_output_filename} ...", end="", flush=True)
-
 		# Now add epub2 compatibility.
 
 		# Include compatibility CSS
@@ -173,12 +164,13 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 		# Simplify the CSS first.  Later we'll update the document to match our simplified selectors.
 		# While we're doing this, we store the original css into a single variable so we can extract the original selectors later.
 		for root, _, filenames in os.walk(work_epub_root_directory):
-			for filename in fnmatch.filter(filenames, "*.css"):
-				with open(Path(root) / filename, "r+", encoding="utf-8") as file:
+			for filename_string in fnmatch.filter(filenames, "*.css"):
+				filename = Path(root) / filename_string
+				with open(filename, "r+", encoding="utf-8") as file:
 					css = file.read()
 
 					# Before we do anything, we process a special case in core.css
-					if "core.css" in filename:
+					if filename.name == "core.css":
 						css = regex.sub(r"abbr{.+?}", "", css, flags=regex.DOTALL)
 
 					total_css = total_css + css + "\n"
@@ -207,12 +199,14 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 
 		# Get a list of .xhtml files to simplify
 		for root, _, filenames in os.walk(work_epub_root_directory):
-			for filename in fnmatch.filter(filenames, "*.xhtml"):
+			for filename_string in fnmatch.filter(filenames, "*.xhtml"):
+				filename = (Path(root) / filename_string).resolve()
+
 				# Don't mess with the ToC, since if we have ol/li > first-child selectors we could screw it up
-				if filename == "toc.xhtml":
+				if filename.name == "toc.xhtml":
 					continue
 
-				with open(Path(root) / filename, "r+", encoding="utf-8") as file:
+				with open(filename, "r+", encoding="utf-8") as file:
 					# We have to remove the default namespace declaration from our document, otherwise
 					# xpath won't find anything at all.  See http://stackoverflow.com/questions/297239/why-doesnt-xpath-work-when-processing-an-xhtml-document-with-lxml-in-python
 					xhtml = file.read().replace(" xmlns=\"http://www.w3.org/1999/xhtml\"", "")
@@ -220,7 +214,7 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 					try:
 						tree = etree.fromstring(str.encode(xhtml))
 					except Exception as ex:
-						raise se.InvalidXhtmlException(f"Error parsing XHTML file: `{filename}`\n{ex}")
+						raise se.InvalidXhtmlException(f"Error parsing XHTML file: [path][link=file://{filename}]{filename}[/][/]. Exception: {ex}")
 
 					# Now iterate over each CSS selector and see if it's used in any of the files we found
 					for selector in selectors:
@@ -235,7 +229,7 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 
 									replacement_class = split_selector[1].replace(":", "").replace("(", "-").replace("n-", "n-minus-").replace("n+", "n-plus-").replace(")", "")
 									selector = selector.replace(split_selector[1], "." + replacement_class, 1)
-									sel = lxml.cssselect.CSSSelector(target_element_selector, translator="xhtml", namespaces=se.XHTML_NAMESPACES)
+									sel = se.easy_xml.css_selector(target_element_selector)
 									for element in tree.xpath(sel.path, namespaces=se.XHTML_NAMESPACES):
 										current_class = element.get("class")
 										if current_class is not None and replacement_class not in current_class:
@@ -249,12 +243,12 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 							# This gets thrown if we use pseudo-elements, which lxml doesn't support
 							pass
 						except lxml.cssselect.SelectorSyntaxError as ex:
-							raise se.InvalidCssException(f"Couldn’t parse CSS in or near this line: `{selector}`\n{ex}")
+							raise se.InvalidCssException(f"Couldn’t parse CSS in or near this line: [css]{selector}[/]. Exception: {ex}")
 
 						# We've already replaced attribute/namespace selectors with classes in the CSS, now add those classes to the matching elements
 						if "[epub|type" in selector:
 							for namespace_selector in regex.findall(r"\[epub\|type\~\=\"[^\"]*?\"\]", selector):
-								sel = lxml.cssselect.CSSSelector(namespace_selector, translator="xhtml", namespaces=se.XHTML_NAMESPACES)
+								sel = se.easy_xml.css_selector(namespace_selector)
 
 								for element in tree.xpath(sel.path, namespaces=se.XHTML_NAMESPACES):
 									new_class = regex.sub(r"^\.", "", se.formatting.namespace_to_class(namespace_selector))
@@ -272,12 +266,12 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 
 					for selector in selectors:
 						try:
-							sel = lxml.cssselect.CSSSelector(selector, translator="xhtml", namespaces=se.XHTML_NAMESPACES)
+							sel = se.easy_xml.css_selector(selector)
 						except lxml.cssselect.ExpressionError:
 							# This gets thrown if we use pseudo-elements, which lxml doesn't support
 							continue
 						except lxml.cssselect.SelectorSyntaxError as ex:
-							raise se.InvalidCssException(f"Couldn’t parse CSS in or near this line: `{selector}`\n{ex}")
+							raise se.InvalidCssException(f"Couldn’t parse CSS in or near this line: [css]{selector}[/]. Exception: {ex}")
 
 						# Convert <abbr> to <span>
 						if "abbr" in selector:
@@ -318,7 +312,7 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 		# Extract cover and cover thumbnail
 		cover_svg_file = work_epub_root_directory / "epub" / "images" / "cover.svg"
 		if not os.path.isfile(cover_svg_file):
-			raise se.MissingDependencyException("Cover image is missing. Did you run `se build-images`?")
+			raise se.MissingDependencyException("Cover image is missing. Did you run [bash]se build-images[/]?")
 
 		svg2png(url=str(cover_svg_file), write_to=str(work_directory / "cover.png"))
 		cover = Image.open(work_directory / "cover.png")
@@ -341,30 +335,32 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 		cover_svg_file.unlink()
 
 		# Massage image references in content.opf
-		metadata_xhtml = metadata_xhtml.replace("cover.svg", "cover.jpg")
-		metadata_xhtml = metadata_xhtml.replace(".svg", ".png")
-		metadata_xhtml = metadata_xhtml.replace("id=\"cover.jpg\" media-type=\"image/svg+xml\"", "id=\"cover.jpg\" media-type=\"image/jpeg\"")
-		metadata_xhtml = metadata_xhtml.replace("image/svg+xml", "image/png")
-		metadata_xhtml = regex.sub(r"properties=\"([^\"]*?)svg([^\"]*?)\"", "properties=\"\\1\\2\"", metadata_xhtml) # We may also have the `mathml` property
+		metadata_xml = metadata_xml.replace("cover.svg", "cover.jpg")
+		metadata_xml = metadata_xml.replace(".svg", ".png")
+		metadata_xml = metadata_xml.replace("id=\"cover.jpg\" media-type=\"image/svg+xml\"", "id=\"cover.jpg\" media-type=\"image/jpeg\"")
+		metadata_xml = metadata_xml.replace("image/svg+xml", "image/png")
+		metadata_xml = regex.sub(r"properties=\"([^\"]*?)svg([^\"]*?)\"", "properties=\"\\1\\2\"", metadata_xml) # We may also have the `mathml` property
 
 		# Add an element noting the version of the se tools that built this ebook
-		metadata_xhtml = regex.sub(r"<dc:publisher", f"<meta property=\"se:built-with\">{se.VERSION}</meta>\n\t\t<dc:publisher", metadata_xhtml)
+		metadata_xml = regex.sub(r"<dc:publisher", f"<meta property=\"se:built-with\">{se.VERSION}</meta>\n\t\t<dc:publisher", metadata_xml)
 
 		# Google Play Books chokes on https XML namespace identifiers (as of at least 2017-07)
-		metadata_xhtml = metadata_xhtml.replace("https://standardebooks.org/vocab/1.0", "http://standardebooks.org/vocab/1.0")
+		metadata_xml = metadata_xml.replace("https://standardebooks.org/vocab/1.0", "http://standardebooks.org/vocab/1.0")
 
 		# Output the modified content.opf so that we can build the kobo book before making more epub2 compatibility hacks
 		with open(work_epub_root_directory / "epub" / "content.opf", "w", encoding="utf-8") as file:
-			file.write(metadata_xhtml)
+			file.write(metadata_xml)
 			file.truncate()
 
 		# Recurse over xhtml files to make some compatibility replacements
 		for root, _, filenames in os.walk(work_epub_root_directory):
-			for filename in filenames:
-				if filename.lower().endswith(".svg"):
+			for filename_string in filenames:
+				filename = Path(root) / filename_string
+
+				if filename.suffix == ".svg":
 					# For night mode compatibility, give the titlepage a 1px white stroke attribute
-					if filename.lower() == "titlepage.svg" or filename.lower() == "logo.svg":
-						with open(Path(root) / filename, "r+", encoding="utf-8") as file:
+					if filename.name in("titlepage.svg", "logo.svg"):
+						with open(filename, "r+", encoding="utf-8") as file:
 							svg = file.read()
 							paths = svg
 
@@ -373,7 +369,7 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 							# so the 2px stroke becomes a 1px stroke that's *outside* of the path instead of being *centered* on the path border.
 							# This looks much nicer, but we also have to increase the image size by 2px in both directions, and re-center the whole thing.
 
-							if filename.lower() == "titlepage.svg":
+							if filename.name == "titlepage.svg":
 								stroke_width = SVG_TITLEPAGE_OUTER_STROKE_WIDTH
 							else:
 								stroke_width = SVG_OUTER_STROKE_WIDTH
@@ -411,11 +407,11 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 
 					# Convert SVGs to PNGs at 2x resolution
 					# Path arguments must be cast to string
-					svg2png(url=str(Path(root) / filename), write_to=regex.sub(r"\.svg$", ".png", str(Path(root) / filename)), scale=2)
-					(Path(root) / filename).unlink()
+					svg2png(url=str(filename), write_to=regex.sub(r"\.svg$", ".png", str(filename)), scale=2)
+					(filename).unlink()
 
-				if filename.lower().endswith(".xhtml"):
-					with open(Path(root) / filename, "r+", encoding="utf-8") as file:
+				if filename.suffix == ".xhtml":
+					with open(filename, "r+", encoding="utf-8") as file:
 						xhtml = file.read()
 						processed_xhtml = xhtml
 
@@ -426,7 +422,7 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 						# Kobo supports presentational MathML. After we build kobo, we convert the presentational MathML to PNG for the rest of the builds.
 						mathml_transform = None
 						for line in regex.findall(r"<(?:m:)?math[^>]*?>(.+?)</(?:m:)?math>", processed_xhtml, flags=regex.DOTALL):
-							mathml_content_tree = se.easy_xml.EasyXmlTree("<?xml version=\"1.0\" encoding=\"utf-8\"?><math xmlns=\"http://www.w3.org/1998/Math/MathML\">{}</math>".format(regex.sub(r"<(/?)m:", "<\\1", line)))
+							mathml_content_tree = se.easy_xml.EasyXhtmlTree("<?xml version=\"1.0\" encoding=\"utf-8\"?><math xmlns=\"http://www.w3.org/1998/Math/MathML\">{}</math>".format(regex.sub(r"<(/?)m:", "<\\1", line)))
 
 							# Initialize the transform object, if we haven't yet
 							if not mathml_transform:
@@ -441,7 +437,7 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 							# Plop our string back in to the XHTML we're processing
 							processed_xhtml = regex.sub(r"<math[^>]*?>\{}\</math>".format(regex.escape(line)), mathml_presentation_xhtml, processed_xhtml, flags=regex.MULTILINE)
 
-						if filename == "endnotes.xhtml":
+						if filename.name == "endnotes.xhtml":
 							# iOS renders the left-arrow-hook character as an emoji; this fixes it and forces it to render as text.
 							# See https://github.com/standardebooks/tools/issues/73
 							# See http://mts.io/2015/04/21/unicode-symbol-render-text-emoji/
@@ -455,7 +451,7 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 						# For example, epilogue can't apply to <article>
 						processed_xhtml = regex.sub(r"<article ([^>]*?)role=\"doc-epilogue\"", "<article \\1", processed_xhtml)
 
-						if filename == "toc.xhtml":
+						if filename.name == "toc.xhtml":
 							landmarks_xhtml = regex.findall(r"<nav epub:type=\"landmarks\">.*?</nav>", processed_xhtml, flags=regex.DOTALL)
 							landmarks_xhtml = regex.sub(r" role=\"doc-.*?\"", "", landmarks_xhtml[0])
 							processed_xhtml = regex.sub(r"<nav epub:type=\"landmarks\">.*?</nav>", landmarks_xhtml, processed_xhtml, flags=regex.DOTALL)
@@ -470,7 +466,7 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 						processed_xhtml = regex.sub(r"class=\"([^\"]*?)epub-type-z3998-publisher-logo([^\"]*?)\"", "class=\"\\1epub-type-z3998-publisher-logo epub-type-se-image-color-depth-black-on-transparent\\2\"", processed_xhtml)
 
 						# Special case for the titlepage
-						if filename == "titlepage.xhtml":
+						if filename.name == "titlepage.xhtml":
 							processed_xhtml = processed_xhtml.replace("<img", "<img class=\"epub-type-se-image-color-depth-black-on-transparent\" epub:type=\"se:image.color-depth.black-on-transparent\"")
 
 						# Google Play Books chokes on https XML namespace identifiers (as of at least 2017-07)
@@ -511,8 +507,8 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 							file.write(processed_xhtml)
 							file.truncate()
 
-				if filename.lower().endswith(".css"):
-					with open(Path(root) / filename, "r+", encoding="utf-8") as file:
+				if filename.suffix == ".css":
+					with open(filename, "r+", encoding="utf-8") as file:
 						css = file.read()
 						processed_css = css
 
@@ -535,8 +531,8 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 
 				for root, _, filenames in os.walk(kobo_work_directory):
 					# Add a note to content.opf indicating this is a transform build
-					for filename in fnmatch.filter(filenames, "content.opf"):
-						with open(Path(root) / filename, "r+", encoding="utf-8") as file:
+					for filename_string in fnmatch.filter(filenames, "content.opf"):
+						with open(Path(root) / filename_string, "r+", encoding="utf-8") as file:
 							xhtml = file.read()
 
 							xhtml = regex.sub(r"<dc:publisher", "<meta property=\"se:transform\">kobo</meta>\n\t\t<dc:publisher", xhtml)
@@ -547,27 +543,34 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 
 					# Kobo .kepub files need each clause wrapped in a special <span> tag to enable highlighting.
 					# Do this here. Hopefully Kobo will get their act together soon and drop this requirement.
-					for filename in fnmatch.filter(filenames, "*.xhtml"):
+					for filename_string in fnmatch.filter(filenames, "*.xhtml"):
 						kobo.paragraph_counter = 1
 						kobo.segment_counter = 1
 
+						filename = (Path(root) / filename_string).resolve()
+
 						# Don't add spans to the ToC
-						if filename == "toc.xhtml":
+						if filename.name == "toc.xhtml":
 							continue
 
-						with open(Path(root) / filename, "r+", encoding="utf-8") as file:
+						with open(filename, "r+", encoding="utf-8") as file:
 							xhtml = file.read()
-							# Kobos don't have fonts that support the ↩ character in endnotes, so replace it with «
-							if filename == "endnotes.xhtml":
+
+							# Note: Kobo supports CSS hyphenation, but it can be improved with soft hyphens.
+							# However we can't insert them, because soft hyphens break the dictionary search when
+							# a word is highlighted.
+
+							# Kobos don't have fonts that support the ↩ character in endnotes, so replace it with ←
+							if filename.name == "endnotes.xhtml":
 								# Note that we replaced ↩ with \u21a9\ufe0e in an earlier iOS compatibility fix
-								xhtml = regex.sub(r"epub:type=\"backlink\">\u21a9\ufe0e</a>", "epub:type=\"backlink\">«</a>", xhtml)
+								xhtml = regex.sub(r"epub:type=\"backlink\">\u21a9\ufe0e</a>", "epub:type=\"backlink\">←</a>", xhtml)
 
 							# We have to remove the default namespace declaration from our document, otherwise
 							# xpath won't find anything at all.  See http://stackoverflow.com/questions/297239/why-doesnt-xpath-work-when-processing-an-xhtml-document-with-lxml-in-python
 							try:
 								tree = etree.fromstring(str.encode(xhtml.replace(" xmlns=\"http://www.w3.org/1999/xhtml\"", "")))
 							except Exception as ex:
-								raise se.InvalidXhtmlException(f"Error parsing XHTML file: `{filename}`\n{ex}", verbose)
+								raise se.InvalidXhtmlException(f"Error parsing XHTML file: [path][link=file://{filename}]{filename}[/][/]. Exception: {ex}")
 
 							kobo.add_kobo_spans_to_node(tree.xpath("./body", namespaces=se.XHTML_NAMESPACES)[0])
 
@@ -583,17 +586,15 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 
 				se.epub.write_epub(kobo_work_directory, output_directory / kobo_output_filename)
 
-			if verbose:
-				print(" OK")
-				print(f"\tBuilding {epub_output_filename} ...", end="", flush=True)
-
 		# Now work on more epub2 compatibility
 
 		# Recurse over css files to make some compatibility replacements.
 		for root, _, filenames in os.walk(work_epub_root_directory):
-			for filename in filenames:
-				if filename.lower().endswith(".css"):
-					with open(Path(root) / filename, "r+", encoding="utf-8") as file:
+			for filename_string in filenames:
+				filename = Path(root) / filename_string
+
+				if filename.suffix == ".css":
+					with open(filename, "r+", encoding="utf-8") as file:
 						css = file.read()
 						processed_css = css
 
@@ -607,7 +608,7 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 							file.truncate()
 
 		# Sort out MathML compatibility
-		has_mathml = "mathml" in metadata_xhtml
+		has_mathml = "mathml" in metadata_xml
 		if has_mathml:
 			# We import this late because we don't want to load selenium if we're not going to use it!
 			from se import browser # pylint: disable=import-outside-toplevel
@@ -620,9 +621,10 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 
 				mathml_count = 1
 				for root, _, filenames in os.walk(work_epub_root_directory):
-					for filename in filenames:
-						if filename.lower().endswith(".xhtml"):
-							with open(Path(root) / filename, "r+", encoding="utf-8") as file:
+					for filename_string in filenames:
+						filename = Path(root) / filename_string
+						if filename.suffix == ".xhtml":
+							with open(filename, "r+", encoding="utf-8") as file:
 								xhtml = file.read()
 								processed_xhtml = xhtml
 								replaced_mathml: List[str] = []
@@ -633,7 +635,7 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 								for line in regex.findall(r"<(?:m:)math[^>]*?>(?:.+?)</(?:m:)math>", processed_xhtml, flags=regex.DOTALL):
 									if line not in replaced_mathml:
 										replaced_mathml.append(line) # Store converted lines to save time in case we have multiple instances of the same MathML
-										mathml_tree = se.easy_xml.EasyXmlTree("<?xml version=\"1.0\" encoding=\"utf-8\"?>{}".format(regex.sub(r"<(/?)m:", "<\\1", line)))
+										mathml_tree = se.easy_xml.EasyXhtmlTree("<?xml version=\"1.0\" encoding=\"utf-8\"?>{}".format(regex.sub(r"<(/?)m:", "<\\1", line)))
 										processed_line = line
 
 										# If the mfenced element has more than one child, they are separated by commas when rendered.
@@ -685,30 +687,31 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 					pass
 
 		# Include epub2 cover metadata
-		cover_id = metadata_tree.xpath("//opf:item[@properties=\"cover-image\"]/@id")[0].replace(".svg", ".jpg")
-		metadata_xhtml = regex.sub(r"(<metadata[^>]+?>)", f"\\1\n\t\t<meta content=\"{cover_id}\" name=\"cover\" />", metadata_xhtml)
+		cover_id = self.metadata_dom.xpath("//item[@properties=\"cover-image\"]/@id")[0].replace(".svg", ".jpg")
+		metadata_xml = regex.sub(r"(<metadata[^>]+?>)", f"\\1\n\t\t<meta content=\"{cover_id}\" name=\"cover\" />", metadata_xml)
 
 		# Add metadata to content.opf indicating this file is a Standard Ebooks compatibility build
-		metadata_xhtml = metadata_xhtml.replace("<dc:publisher", "<meta property=\"se:transform\">compatibility</meta>\n\t\t<dc:publisher")
+		metadata_xml = metadata_xml.replace("<dc:publisher", "<meta property=\"se:transform\">compatibility</meta>\n\t\t<dc:publisher")
 
 		# Add any new MathML images we generated to the manifest
 		if has_mathml:
 			for root, _, filenames in os.walk(work_epub_root_directory / "epub" / "images"):
-				filenames = se.natural_sort(filenames)
+				filenames = natsorted(filenames)
 				filenames.reverse()
-				for filename in filenames:
-					if filename.lower().startswith("mathml-"):
-						metadata_xhtml = metadata_xhtml.replace("<manifest>", f"<manifest><item href=\"images/{filename}\" id=\"{filename}\" media-type=\"image/png\"/>")
+				for filename_string in filenames:
+					filename = Path(root) / filename_string
+					if filename.name.startswith("mathml-"):
+						metadata_xml = metadata_xml.replace("<manifest>", f"<manifest><item href=\"images/{filename.name}\" id=\"{filename.name}\" media-type=\"image/png\"/>")
 
-			metadata_xhtml = regex.sub(r"properties=\"([^\"]*?)mathml([^\"]*?)\"", "properties=\"\\1\\2\"", metadata_xhtml)
+			metadata_xml = regex.sub(r"properties=\"([^\"]*?)mathml([^\"]*?)\"", "properties=\"\\1\\2\"", metadata_xml)
 
-		metadata_xhtml = regex.sub(r"properties=\"\s*\"", "", metadata_xhtml)
+		metadata_xml = regex.sub(r"properties=\"\s*\"", "", metadata_xml)
 
 		# Generate our NCX file for epub2 compatibility.
 		# First find the ToC file.
-		toc_filename = metadata_tree.xpath("//opf:item[@properties=\"nav\"]/@href")[0]
-		metadata_xhtml = metadata_xhtml.replace("<spine>", "<spine toc=\"ncx\">")
-		metadata_xhtml = metadata_xhtml.replace("<manifest>", "<manifest><item href=\"toc.ncx\" id=\"ncx\" media-type=\"application/x-dtbncx+xml\" />")
+		toc_filename = self.metadata_dom.xpath("//item[@properties=\"nav\"]/@href")[0]
+		metadata_xml = metadata_xml.replace("<spine>", "<spine toc=\"ncx\">")
+		metadata_xml = metadata_xml.replace("<manifest>", "<manifest><item href=\"toc.ncx\" id=\"ncx\" media-type=\"application/x-dtbncx+xml\" />")
 
 		# Now use an XSLT transform to generate the NCX
 		with importlib_resources.path("se.data", "navdoc2ncx.xsl") as navdoc2ncx_xsl_filename:
@@ -716,7 +719,7 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 
 		# Convert the <nav> landmarks element to the <guide> element in content.opf
 		guide_xhtml = "<guide>"
-		for element in toc_tree.xpath("//xhtml:nav[@epub:type=\"landmarks\"]/xhtml:ol/xhtml:li/xhtml:a"):
+		for element in toc_tree.xpath("//nav[@epub:type=\"landmarks\"]/ol/li/a"):
 			element_xhtml = element.tostring()
 			element_xhtml = regex.sub(r"epub:type=\"([^\"]*)(\s*frontmatter\s*|\s*backmatter\s*)([^\"]*)\"", "type=\"\\1\\3\"", element_xhtml)
 			element_xhtml = regex.sub(r"epub:type=\"[^\"]*(acknowledgements|bibliography|colophon|copyright-page|cover|dedication|epigraph|foreword|glossary|index|loi|lot|notes|preface|bodymatter|titlepage|toc)[^\"]*\"", "type=\"\\1\"", element_xhtml)
@@ -736,53 +739,48 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 
 		guide_xhtml = guide_xhtml + "</guide>"
 
-		metadata_xhtml = metadata_xhtml.replace("</package>", "") + guide_xhtml + "</package>"
+		metadata_xml = metadata_xml.replace("</package>", "") + guide_xhtml + "</package>"
 
 		# Guide is done, now write content.opf and clean it.
 		# Output the modified content.opf before making more epub2 compatibility hacks.
 		with open(work_epub_root_directory / "epub" / "content.opf", "w", encoding="utf-8") as file:
-			file.write(metadata_xhtml)
+			file.write(metadata_xml)
 			file.truncate()
 
 		# All done, clean the output
 		for filepath in se.get_target_filenames([work_epub_root_directory], (".xhtml", ".svg", ".opf", ".ncx")):
-			se.formatting.format_xhtml_file(filepath, False, filepath.name == "content.opf", filepath.name == "endnotes.xhtml", filepath.name == "colophon.xhtml")
+			se.formatting.format_xml_file(filepath)
 
 		# Write the compatible epub
 		se.epub.write_epub(work_epub_root_directory, output_directory / epub_output_filename)
 
-		if verbose:
-			print(" OK")
-
 		if run_epubcheck:
-			if verbose:
-				print(f"\tRunning epubcheck on {epub_output_filename} ...", end="", flush=True)
-
 			# Path arguments must be cast to string for Windows compatibility.
 			with importlib_resources.path("se.data.epubcheck", "epubcheck.jar") as jar_path:
-				output = subprocess.run(["java", "-jar", str(jar_path), "--quiet", str(output_directory / epub_output_filename)], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, check=False).stdout.decode().strip()
-
-				if output:
+				try:
+					epubcheck_result = subprocess.run(["java", "-jar", str(jar_path), "--quiet", str(output_directory / epub_output_filename)], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, check=False)
+					epubcheck_result.check_returncode()
+				except subprocess.CalledProcessError:
+					output = epubcheck_result.stdout.decode().strip()
 					# Get the epubcheck version to print to the console
 					version_output = subprocess.run(["java", "-jar", str(jar_path), "--version"], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, check=False).stdout.decode().strip()
 					version = regex.search(r"[0-9]+\.([0-9]+\.?)*", version_output, flags=regex.MULTILINE).group(0)
 
-					# Remove trailing lines from epubcheck output
-					output = output.replace("\n\nCheck finished with errors", "")
+					# The last two lines from epubcheck output are not necessary. Remove them here.
+					# Remove them as lines instead of as a matching regex to work with localized output strings.
+					split_output = output.split("\n")
+					output = "\n".join(split_output[:-2])
 
-					if verbose:
-						print(f"\n\t\tepubcheck v{version} failed with:\n\t\t" + "\t\t".join(output.splitlines(True)), file=sys.stderr)
-					else:
-						print(f"epubcheck v{version} failed with:\n{output}", file=sys.stderr)
-					return
+					# Try to linkify files in output if we can find them
+					try:
+						output = regex.sub(r"(ERROR\(.+?\): )(.+?)(\([0-9]+,[0-9]+\))", lambda match: match.group(1) + "[path][link=file://" + str(self.path / "src" / regex.sub(fr"^\..+?\.epub{os.sep}", "", match.group(2))) + "]" + match.group(2) + "[/][/]" + match.group(3), output)
+					except:
+						# If something goes wrong, just pass through the usual output
+						pass
 
-			if verbose:
-				print(" OK")
+					raise se.BuildFailedException(f"[bash]epubcheck[/] v{version} failed with:\n{output}")
 
 		if build_kindle:
-			if verbose:
-				print(f"\tBuilding {kindle_output_filename} ...", end="", flush=True)
-
 			# There's a bug in Calibre <= 3.48.0 where authors who have more than one MARC relator role
 			# display as "unknown author" in the Kindle interface.
 			# See: https://bugs.launchpad.net/calibre/+bug/1844578
@@ -822,11 +820,11 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 
 			# Clean just the ToC and NCX
 			for filepath in [work_epub_root_directory / "epub" / "toc.ncx", work_epub_root_directory / "epub" / toc_filename]:
-				se.formatting.format_xhtml_file(filepath, False)
+				se.formatting.format_xml_file(filepath)
 
 			# Convert endnotes to Kindle popup compatible notes
-			if (work_epub_root_directory / "epub" / "text" / "endnotes.xhtml").is_file():
-				with open(work_epub_root_directory / "epub" / "text" / "endnotes.xhtml", "r+", encoding="utf-8") as file:
+			if (work_epub_root_directory / "epub/text/endnotes.xhtml").is_file():
+				with open(work_epub_root_directory / "epub/text/endnotes.xhtml", "r+", encoding="utf-8") as file:
 					xhtml = file.read()
 
 					# We have to remove the default namespace declaration from our document, otherwise
@@ -834,7 +832,7 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 					try:
 						tree = etree.fromstring(str.encode(xhtml.replace(" xmlns=\"http://www.w3.org/1999/xhtml\"", "")))
 					except Exception as ex:
-						raise se.InvalidXhtmlException(f"Error parsing XHTML file: `endnotes.xhtml`\n{ex}")
+						raise se.InvalidXhtmlException(f"Error parsing XHTML [path][link=file://{(work_epub_root_directory / 'epub/text/endnotes.xhtml').resolve()}]endnotes.xhtml[/][/]. Exception: {ex}")
 
 					notes = tree.xpath("//li[@epub:type=\"endnote\" or @epub:type=\"footnote\"]", namespaces=se.XHTML_NAMESPACES)
 
@@ -848,7 +846,7 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 						try:
 							ref_link = etree.tostring(note.xpath("p[last()]/a[last()]")[0], encoding="unicode", pretty_print=True, with_tail=False).replace(" xmlns:epub=\"http://www.idpf.org/2007/ops\"", "").strip()
 						except Exception:
-							raise se.InvalidXhtmlException(f"Can’t find ref link for `#{note_id}`.")
+							raise se.InvalidXhtmlException(f"Can’t find ref link for [url]#{note_id}[/].")
 
 						new_ref_link = regex.sub(r">.*?</a>", ">" + note_number + "</a>.", ref_link)
 
@@ -896,9 +894,10 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 
 			# Do some compatibility replacements
 			for root, _, filenames in os.walk(work_epub_root_directory):
-				for filename in filenames:
-					if filename.lower().endswith(".xhtml"):
-						with open(Path(root) / filename, "r+", encoding="utf-8") as file:
+				for filename_string in filenames:
+					filename = Path(root) / filename_string
+					if filename.suffix == ".xhtml":
+						with open(filename, "r+", encoding="utf-8") as file:
 							xhtml = file.read()
 							processed_xhtml = xhtml
 
@@ -928,13 +927,13 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 
 			# Generate the Kindle file
 			# We place it in the work directory because later we have to update the asin, and the mobi.update_asin() function will write to the final output directory
-			cover_path = work_epub_root_directory / "epub" / metadata_tree.xpath("//opf:item[@properties=\"cover-image\"]/@href")[0].replace(".svg", ".jpg")
+			cover_path = work_epub_root_directory / "epub" / self.metadata_dom.xpath("//item[@properties=\"cover-image\"]/@href")[0].replace(".svg", ".jpg")
 
 			# Path arguments must be cast to string for Windows compatibility.
 			return_code = subprocess.run([str(ebook_convert_path), str(work_directory / epub_output_filename), str(work_directory / kindle_output_filename), "--pretty-print", "--no-inline-toc", "--max-toc-links=0", "--prefer-metadata-cover", f"--cover={cover_path}"], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, check=False).returncode
 
 			if return_code:
-				raise se.InvalidSeEbookException("`ebook-convert` failed.")
+				raise se.InvalidSeEbookException("[bash]ebook-convert[/] failed.")
 
 			# Success, extract the Kindle cover thumbnail
 
@@ -946,6 +945,3 @@ def build(self, metadata_xhtml: str, metadata_tree: se.easy_xml.EasyXmlTree, run
 			kindle_cover_thumbnail = kindle_cover_thumbnail.convert("RGB") # Remove alpha channel from PNG if necessary
 			kindle_cover_thumbnail = kindle_cover_thumbnail.resize((432, 648))
 			kindle_cover_thumbnail.save(output_directory / f"thumbnail_{asin}_EBOK_portrait.jpg")
-
-			if verbose:
-				print(" OK")
