@@ -10,6 +10,25 @@ import se
 
 DICTIONARY: Set[str] = set()	# Store our hyphenation dictionary so we don't re-read the file on every pass
 
+def get_xhtml_language(xhtml: str) -> str:
+	"""
+	Try to get the IETF lang tag for a complete XHTML document
+	"""
+
+	supported_languages = ["en-US", "en-GB", "en-AU", "en-CA"]
+
+	match = regex.search(r"<html[^>]+?xml:lang=\"([^\"]+)\"", xhtml)
+
+	if match:
+		language = match.group(1)
+	else:
+		language = None
+
+	if language not in supported_languages:
+		raise se.InvalidLanguageException(f"No valid [attr]xml:lang[/] attribute in [xhtml]<html>[/] element. Only [text]{'[/], [text]'.join(supported_languages[:-1])}[/], and [text]{supported_languages[-1]}[/] are supported.")
+
+	return language
+
 def modernize_hyphenation(xhtml: str) -> str:
 	"""
 	Convert old-timey hyphenated compounds into single words based on the passed DICTIONARY.
@@ -31,7 +50,7 @@ def modernize_hyphenation(xhtml: str) -> str:
 
 	# The non-capturing group at the beginning tries to prevent
 	# bad matches like stag's-horn -> stag'shorn or dog's-eared -> dog'seared
-	result = regex.findall(r"(?:[^’\'])\b[^\W\d_]+\-[^\W\d_]+\b", xhtml)
+	result = regex.findall(r"(?<![’\'])\b[^\W\d_]+\-[^\W\d_]+\b", xhtml)
 
 	for word in set(result): # set() removes duplicates
 		new_word = word.replace("-", "").lower()
@@ -40,7 +59,7 @@ def modernize_hyphenation(xhtml: str) -> str:
 			# then replace the original match with them joined together and titlecased.
 			lhs = regex.sub(r"\-.+$", r"", word)
 			rhs = regex.sub(r"^.+?\-", r"", word)
-			xhtml = regex.sub(r"" + lhs + "-" + rhs, lhs + rhs.lower(), xhtml)
+			xhtml = regex.sub(fr"{lhs}-{rhs}", lhs + rhs.lower(), xhtml)
 
 	# Quick fix for a common error cases
 	xhtml = xhtml.replace("z3998:nonfiction", "z3998:non-fiction")
@@ -66,7 +85,7 @@ def detect_problem_spellings(xhtml: str) -> list:
 	"""
 
 	# Uncomment if we eventually need the document language
-	# language = se.get_xhtml_language(xhtml)
+	# language = get_xhtml_language(xhtml)
 	output = []
 
 	if regex.search(r"\bstaid\b", xhtml):
@@ -84,6 +103,9 @@ def detect_problem_spellings(xhtml: str) -> list:
 	if regex.search(r"\bmann?ikin\b", xhtml):
 		output.append("“mannikin” or “manikin” detected. Confirm that “mannikin” is used in the sense of a small person, and “mannequin” is used in the sense of a dummy or figure.")
 
+	if regex.search(r"\bgripe", xhtml):
+		output.append("“gripe” or “griped” detected. Confirm that “gripe” is used in the sense of illness or complaint, not in the sense of “grip” or “gripped.”")
+
 	return output
 
 def modernize_spelling(xhtml: str) -> str:
@@ -97,7 +119,7 @@ def modernize_spelling(xhtml: str) -> str:
 	A string representing the XHTML with its spelling modernized
 	"""
 
-	language = se.get_xhtml_language(xhtml)
+	language = get_xhtml_language(xhtml)
 
 	# ADDING NEW WORDS TO THIS LIST:
 	# A good way to check if a word is "archaic" is to do a Google N-Gram search: https://books.google.com/ngrams/graph?case_insensitive=on&year_start=1800&year_end=2000&smoothing=3
@@ -108,13 +130,14 @@ def modernize_spelling(xhtml: str) -> str:
 	xhtml = regex.sub(r"\b([Oo])ker\b", r"\1cher", xhtml)				# oker -> ocher
 	xhtml = regex.sub(r"\b([Ww])ellnigh\b", r"\1ell-nigh", xhtml)			# wellnigh -> well-nigh
 	xhtml = regex.sub(r"\b([Tt]he|[Aa]nd|[Oo]r) what not(?! to)\b", r"\1 whatnot", xhtml)	# what not -> whatnot
-	xhtml = regex.sub(r"\b([Gg])ood\-bye?\b", r"\1oodbye", xhtml)			# good-by -> goodbye
+	xhtml = regex.sub(r"\b([Gg])ood[\-]bye?\b", r"\1oodbye", xhtml)			# good-by -> goodbye
+	xhtml = regex.sub(r"\b([Gg])ood\sbye\b", r"\1oodbye", xhtml)			# good bye -> goodbye (Note that we can't do `good by` -> `goodby` because one might do good by someone.
 	xhtml = regex.sub(r"\b([Hh])ind(u|oo)stanee", r"\1industani", xhtml)		# hindoostanee -> hindustani
 	xhtml = regex.sub(r"\b([Hh])indoo", r"\1indu", xhtml)				# hindoo -> hindu
 	xhtml = regex.sub(r"\b([Ee])xpence", r"\1xpense", xhtml)			# expence -> expense
 	xhtml = regex.sub(r"\b([Ll])otos", r"\1otus", xhtml)				# lotos -> lotus
 	xhtml = regex.sub(r"\b([Ss])collop", r"\1callop", xhtml)			# scollop -> scallop
-	xhtml = regex.sub(r"\b([Ss])ubtil(?!(ize|izing))", r"\1ubtle", xhtml)		# subtil -> subtle (but "subtilize" and "subtilizing")
+	xhtml = regex.sub(r"\b([Ss])ubtile?(?!(ize|izing))", r"\1ubtle", xhtml)		# subtil -> subtle (but "subtilize" and "subtilizing")
 	xhtml = regex.sub(r"\bQuoiff", r"Coif", xhtml)					# quoiff -> coif
 	xhtml = regex.sub(r"\bquoiff", r"coif", xhtml)					# quoiff -> coif
 	xhtml = regex.sub(r"\bIndorse", r"Endorse", xhtml)				# indorse -> endorse
@@ -152,7 +175,6 @@ def modernize_spelling(xhtml: str) -> str:
 	xhtml = regex.sub(r"\b([Cc])oup\-de\-grace", r"\1oup-de-grâce", xhtml)		# coup-de-grace -> coup-de-grâce
 	xhtml = regex.sub(r"\b([Cc])anape", r"\1anapé", xhtml)				# canape -> canapé
 	xhtml = regex.sub(r"\b([Pp])recis\b", r"\1récis", xhtml)			# precis -> précis
-	xhtml = regex.sub(r"\b([Gg])ood\-by([^e])", r"\1oodbye\2", xhtml)		# good-by -> goodbye
 	xhtml = regex.sub(r"\b([Gg])ood\-night", r"\1ood night", xhtml)			# good-night -> good night
 	xhtml = regex.sub(r"\b([Gg])ood\-morning", r"\1ood morning", xhtml)		# good-morning -> good morning
 	xhtml = regex.sub(r"\b([Gg])ood\-evening", r"\1ood evening", xhtml)		# good-evening -> good evening
@@ -164,7 +186,7 @@ def modernize_spelling(xhtml: str) -> str:
 	xhtml = regex.sub(r"\ba la\b", r"à la", xhtml)					# a la -> à la
 	xhtml = regex.sub(r"\ba propos\b", r"apropos", xhtml)				# a propos -> apropos
 	xhtml = regex.sub(r"\bper cent(s?)\b", r"percent\1", xhtml)			# per cent -> percent
-	xhtml = regex.sub(r"\bpercent\.(\s+[a-z])", r"percent\1", xhtml)		# percent. followed by lowercase -> percent
+	xhtml = regex.sub(r"\bpercent\.(\s+[\p{Lowercase_Letter}])", r"percent\1", xhtml)		# percent. followed by lowercase -> percent
 	xhtml = regex.sub(r"\bpercent\.,\b", r"percent,", xhtml)			# per cent. -> percent
 	xhtml = regex.sub(r"\b([Ee])ntree(s?)\b", r"\1ntrée\2", xhtml)			# entree -> entrée
 	xhtml = regex.sub(r"\b([Ff])iance", r"\1iancé", xhtml)				# fiance -> fiancé
@@ -246,10 +268,11 @@ def modernize_spelling(xhtml: str) -> str:
 	xhtml = regex.sub(r"\b([Tt])abu(s?)\b", r"\1aboo\2", xhtml)			# tabu -> taboo
 	xhtml = regex.sub(r"\b([Kk])idnaping\b", r"\1idnapping", xhtml)			# kidnaping -> kidnapping
 	xhtml = regex.sub(r"\bQuixotic\b", r"quixotic", xhtml)				# Quixotic -> quixotic
-	xhtml = regex.sub(r"([^a-z]’[Tt])\s(is|were|was)\b", r"\1\2", xhtml)		# 't is, 't was, 't were -> 'tis, 'twas, 'twere
+	xhtml = regex.sub(r"([^\p{Lowercase_Letter}]’[Tt])\s(is|were|was|isn’t)\b", r"\1\2", xhtml)		# 't is, 't was, 't were 't isn't -> 'tis, 'twas, 'twere, 't isn't
 	xhtml = regex.sub(r"\b([Uu])p stairs\b", r"\1pstairs", xhtml)			# up stairs -> upstairs
 	xhtml = regex.sub(r"(?<!up and )(?<!up or )\b([Dd])own stairs\b", r"\1ownstairs", xhtml)		# down stairs -> downstairs, but not "up (or|and) down stairs"
 	xhtml = regex.sub(r"([Pp])artizan", r"\1artisan", xhtml)			# partizan -> partisan
+	xhtml = regex.sub(r"([Nn])onplused", r"\1onplussed", xhtml)			# nonplused -> nonplussed
 
 	# Normalize some names
 	xhtml = regex.sub(r"Moliere", r"Molière", xhtml)				# Moliere -> Molière
@@ -273,6 +296,10 @@ def modernize_spelling(xhtml: str) -> str:
 	xhtml = regex.sub(r"\bTimbuctoo", r"Timbuktu", xhtml)				# Timbuctoo -> Timbuktu
 	xhtml = regex.sub(r"\bRumania", r"Romania", xhtml)				# Rumania -> Romania
 	xhtml = regex.sub(r"\bTokio", r"Tokyo", xhtml)					# Tokio -> Tokyo
+	xhtml = regex.sub(r"\bTchekh?ov", r"Chekhov", xhtml)				# Tchekhov -> Chekhov
+	xhtml = regex.sub(r"\bVereshtchagin", r"Vereshchagin", xhtml)			# Vereshtchagin -> Vereshchagin
+	xhtml = regex.sub(r"\bSoudan", "Sudan", xhtml)					# Soudan -> Sudan
+	xhtml = regex.sub(r"\bJack-in-the-box", "jack-in-the-box", xhtml)		# Jack-in-the-box -> jack-in-the-box
 
 	# Remove archaic diphthongs
 	xhtml = regex.sub(r"\b([Mm])edi(æ|ae)val", r"\1edieval", xhtml)
@@ -304,8 +331,11 @@ def modernize_spelling(xhtml: str) -> str:
 
 	# US spelling is unique
 	if language == "en-US":
-		xhtml = regex.sub(r"\b([Mm])anœuve?r", r"\1aneuver", xhtml) # Omit last letter to catch both maneuverS and maneuverING
+		xhtml = regex.sub(r"\b([Mm])anœuv(?:er|re)", r"\1aneuver", xhtml) # Omit last letter to catch both maneuverS and maneuverING
+		xhtml = regex.sub(r"\b([Mm])anœuvering", r"\1aneuvering", xhtml)
 	else:
-		xhtml = regex.sub(r"\b([Mm])anœuve?r", r"\1anoeuvr", xhtml) # Omit last letter to catch both manoeuvreS and manoeuvrING
+		xhtml = regex.sub(r"\b([Mm])anœuv(?:er|re)", r"\1anoeuvre", xhtml)
+		xhtml = regex.sub(r"\b([Mm])anœuvring", r"\1anoeuvring", xhtml)
+		xhtml = regex.sub(r"\b([Mm])anoeuvreing", r"\1anoeuvring", xhtml)
 
 	return xhtml
